#!/usr/bin/perl

package SF::PacketView;

# -- See .pre-flyloader for original file
use FlyLoader;
use strict;
use File::Temp qw(tempfile tempdir);
use Data::Dumper;
use Error qw(:try);
use Date::Manip;
use POSIX;
use URI::Escape;

our $_debug;
our $imsConf = SF::Util::load_ims_conf();
$_debug = $imsConf->{DEBUG_PACKET_VIEW};
our $expandState = SF::UserPreferences::get_preference_value("Events","Expand packet view by default") || 0;

my $csvoutput = 0;
my $hidelinks = 0;

my $imgpath = "/img";

sub setRelativePath
{
  my ($pref) = @_;

  if ($pref)
  {
    $imgpath = "./img";
  }
  else
  {
    $imgpath = "/img";
  }
}
#################################################################################################
#Endace_Integration_Patch									 #
#This work is licensed under the Creative Commons Attribution-ShareAlike 3.0 Unported License.  #
#To view a copy of the license , visit http://creativecommons.org/licenses/by-sa/3.0            #
#################################################################################################
use LWP::UserAgent;
use HTTP::Request;
use HTTP::Response;
use IO::Handle;
use JSON qw( decode_json); 
use DateTime;
use strict;
use warnings;

my $endace_int_version = 3.0; #Endace integration code version dated:2015-04-13 13:52:15 PM EST
my $api_version = 'v1'; # current endace REST API version
my $endace_debug_print = 0; # Debug o/p to SourceFire Web
my $endace_integration_enable=0 ; # Enabled automatically
my $endace_debug = 0; # Debug Print to local temp File

my $endace_configfile = '/etc/sf/endace.conf';
my $viz_indexfile = '/etc/sf/endace_viz_id.conf';
my $OUTFILE = '';

#my $endace_configfile = 'endace.conf';
#my $viz_indexfile = 'vision.conf';

if ($endace_debug == 1)
{
    use Data::Dumper;
    open OUTFILE , '>', "sessiondebug" if $endace_debug;  # Test Debug, Must be disabled in Production
    
    print OUTFILE "\nEndace Config file $endace_configfile\n" if $endace_debug;
    print OUTFILE "\nEndace Vision file $viz_indexfile\n" if $endace_debug;

}

my %visualization_metalinks=(); #Visualization links array
#my %visualizations=(); #Visualization links array
my @vizlinks = ();

sub delolderdatamines
{
    my ($epAddr, $epUser, $epPass, $created_before) = @_;

    my $ua = LWP::UserAgent->new(); 
 
    #set custom HTTP request header fields
    my $req = HTTP::Request->new(GET => "http://$epAddr/api/$api_version/datamines");
    $req->authorization_basic($epUser,$epPass);
    $req->header('content-type' => 'application/json');
    my $resp = $ua->request($req);
    if ($resp->is_success) 
    {
	my $message = $resp->decoded_content;
	my $json_msg = decode_json($message);
	foreach my $datamine (@{$json_msg->{'datamines'}})
	{
	    print OUTFILE "\nTime when this datamine session is created: $datamine->{'timeCreated'}\n" if $endace_debug;
	    if ($created_before < 3600)
	    {
		$created_before = 3600;
                print OUTFILE "\nCreation before time forced to 3600\n" if $endace_debug;
	    }
		
	    my $creation_time =  (time - $created_before);
      	    if ($datamine->{'timeCreated'} < $creation_time)
	    {
	        foreach my $link (@{$datamine->{'links'}})
	        {
		    if ($link->{'name'} eq "deleteDatamine")
		    {
			print OUTFILE "\nDeleting older datamine href: $link->{'href'} created before $creation_time" if $endace_debug;
		
			my $dua = LWP::UserAgent->new(); 
			#set custom HTTP request header fields for POST
			my $dreq = HTTP::Request->new(DELETE => $link->{'href'});
			$dreq->authorization_basic($epUser,$epPass);
			my $dresp = $dua->request($dreq);
			#print OUTFILE "\nDatamine - HTTP DEL code: $resp->code\n" if $endace_debug;
		    }
		}
	    }
	}
    }	
}


sub readconfigfile
{
   #Use the Event's Detection Engine ID mapping to extract a matching Probe/user configuration from endace.conf

    my ($sensor) = @_;
    
    print OUTFILE "\nFunction Name - readingconfigfile\n" if $endace_debug;

    my %endaceconfig = ();
    my %defaultconfig = ();
    my $found_de_match = 0;	

    if ( open (CONFIG,"$endace_configfile") )
    {
        while (my $line = <CONFIG>)  
        {
	    next unless $line =~ /\S/;
	    chomp $line;
	    print OUTFILE $line ,"\n" if $endace_debug;
	    if ((my $de, my $epAddr, my $epUser, my $epPass, my $sBefore, my $sAfter, my $created_before) = split(",", $line))
            {
		if ($de !~ '#' && $de eq $sensor) 
 		{
		    $endaceconfig{$de}{'epAddr'} = $epAddr;
		    $endaceconfig{$de}{'epUser'} = $epUser;
		    $endaceconfig{$de}{'epPass'} = $epPass;
		    $endaceconfig{$de}{'epsBefore'} = $sBefore;
		    $endaceconfig{$de}{'epsAfter'} = $sAfter;
		    $found_de_match = 1;
		    $endace_integration_enable = 1;
		    delolderdatamines($epAddr,$epUser,$epPass,$created_before);
                }
		if ($de !~ '#' && $de eq 'default')
	        {
		    $defaultconfig{$de}{'epAddr'} = $epAddr;
		    $defaultconfig{$de}{'epUser'} = $epUser;
		    $defaultconfig{$de}{'epPass'} = $epPass;
		    $defaultconfig{$de}{'epsBefore'} = $sBefore;
		    $defaultconfig{$de}{'epsAfter'} = $sAfter; 	
		    $endace_integration_enable = 1;
		    delolderdatamines($epAddr,$epUser,$epPass,$created_before);		
	        } 	
		
	    }
	}
   }
   close (CONFIG);
 #  close OUTFILE if $endace_debug;

   if ($found_de_match eq 1)
   {
       return (\%endaceconfig);
   }    
   else 
   {
       return (\%defaultconfig);
   }
 
}

sub getsession
{
    #Extract the Event Variables and Map it to Endace session hash
    
    my ($event ) = @_;
    my %session=( sip => 0,
                dip => 0,
                spt => 0,
                dpt => 0,
                protocol => 0,
                protocolname => 0,
                timestamp => 0,
                eventname => 0,
                ); # Session hash, in it's most simple form.
 
    $session{'timestamp'} = $event->{'tvsec'};
    $session{'eventname'} = $event->{'display_text'};
    

    #open OUTFILE , '>>', "/tmp/sessiondebug" if $endace_debug;  # Test Debug, Must be disabled in Production
    print OUTFILE "\nFunction Name - getsession\n" if $endace_debug;


    #Walk over the columns grabbing the data we need to use for extration.
    foreach my $column (@{$event->{'columns'}}) 
    {
	print OUTFILE $column->{'name'} if $endace_debug;
	print OUTFILE $column->{'data'} if $endace_debug;

        if ($column->{'name'} eq 'src') 
        {
	    if ($column->{'data'} =~ m/^00000000000000000000FFFF.*/) 
	    {# Check if this value looks like a packed v4 address. If so, lets unpack it
                my $val = $column->{'data'};
	        $val =~ s/00000000000000000000FFFF//;
	        $session{'sip'} = join '.', unpack "C*", pack "H*", $val;
	    } 
	    else 
	    {
		#This must be valid ipv6 address
		if ($endace_debug == 1)
		{
		    $session{'sip'} = $column->{'data'};
		}
		else
		{
	            print OUTFILE "src doesn't look IPv4 style\n" if $endace_debug;
		    $session{'sip'} = join ':', unpack "(a4)*", $column->{'data'};
		}
	    }
            print OUTFILE "Decoded src is $session{'sip'}\n" if $endace_debug;
        }

	
        if ($column->{'name'} eq 'dst') 
        {
	    if ($column->{'data'} =~ m/^00000000000000000000FFFF.*/) 
	    {# Check if this value looks like a packed v4 address. If so, lets unpack it
                my $val = $column->{'data'};
	        $val =~ s/00000000000000000000FFFF//;
	        $session{'dip'} = join '.', unpack "C*", pack "H*", $val;
	    } 
	    else 
	    {
		if ($endace_debug == 1)
		{
		    $session{'dip'} = $column->{'data'};
		}
		else
		{
	            #This must be valid ipv6 address
	            print OUTFILE "dst doesn't look IPv4 style\n" if $endace_debug;
		    $session{'dip'} = join ':', unpack "(a4)*", $column->{'data'};
		}
	    }
            print OUTFILE "Decoded dst is $session{'dip'}\n" if $endace_debug;
        }
        
        if ($column->{'name'} eq 'sport')
	{
            ($session{'spt'}, my $proto) = split(/:/, $column->{'data'});
	    print OUTFILE "proto is $proto\n" if $endace_debug;
	    $session{'protocolname'} = $proto;
            #if ($proto == 6) 
	    #{
            #    $session{'protocolname'} = "tcp";
            #    $session{'protocol'} = $proto;
            #} elsif ($proto == 1) 
	    #{
            #    $session{'protocolname'} = "icmp";  
            #    $session{'protocol'} = $proto;
            #} elsif ($proto == 17) 
	    #{
            #    $session{'protocolname'} = "udp";
            #    $session{'protocol'} = $proto;
            #}
            #else
	    #{
	    #	#Other Protocol Numbers, dont know their name
	    #$session{'protocolname'} = $proto;
	    #}
        }

	if ($column->{'name'} eq 'dport')
	{
            ($session{'dpt'}, my $proto) = split(/:/, $column->{'data'});
	    print OUTFILE "proto is $proto\n" if $endace_debug;
	    $session{'protocolname'} = $proto;
            #if ($proto == 6) 
	    #{
            #    $session{'protocolname'} = "tcp";
            #    $session{'protocol'} = $proto;
            #} elsif ($proto == 1) 
	    #{
            #    $session{'protocolname'} = "icmp";  
            #    $session{'protocol'} = $proto;
            #} elsif ($proto == 17) 
	    #{
            #    $session{'protocolname'} = "udp";
            #    $session{'protocol'} = $proto;
            #}
	    #else
	    #{
	    #	#Other Protocol Numbers, dont know their name
	    #	$session{'protocolname'} = $proto;
	    #}
        }

        if ($column->{'name'} eq 'sensor'){
          $session{'sensor'} =  $column->{'data'};
          print "<br><b>Sourcefire DE:</b> $session{'sensor'}</br>" ;
        }
    }
    #close OUTFILE if $endace_debug;
    return(\%session);
}


sub generate_viz_link
{
    my $user_ref = shift;
    my (%user) =  %{$user_ref};
    my $rotid_ref = shift;
    my @rotfids = @{$rotid_ref};
    my $probeip = shift;
    my $eventname = shift;
    
    #login to endace probe
    my $ua = LWP::UserAgent->new();
    $ua->agent("Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/20120427 Firefox/15.0a1");
    $ua->cookie_jar( {} );

    my $req = HTTP::Request->new(GET => "http://$probeip/vision/user/login/");
    $req->method("POST");
    $req->content_type("application/x-www-form-urlencoded");
    $req->content("username=$user{'login'}&password=$user{'pass'}&login_submit=Login");
    my $resp = $ua->request($req);
    
    
    #Get the Visualization Top View Page
    $req = HTTP::Request->new(GET => "http://$probeip/vision/visualizations/");
    $req->method("GET");
    $req->content_type("application/x-www-form-urlencoded");					
    $resp = $ua->request($req);
    my $page = $resp->content; 
    
    #Read the local mapping file 
    my %rdfileidseen = ();
    my @wrfilecontent = ("#ProbeIP,RotFileID,Href");
    my @stale_matched_rotfids = ();
    
    if ( open (VISION,"$viz_indexfile") )
    {
	while (my $line = <VISION>)  
        {
	    next unless $line =~ /\S/;
	    chomp $line;
	    if ((my $pip, my $ruid, my $href) = split(",", $line))
	    {
		if ($pip !~ '#')
		{
		   $rdfileidseen{$ruid} = $line;
		}
	    }
	}
	close VISION;
    }
    
    #print  %rdfileidseen;
    #print "\n"; 
    
    #Search rotid in local file
    # If Rotfile ID is found, extract href.
    # if href is NOT STALE, Then the link is precreated
    # if href is STALE
    # update the local file
    my @newrotids = ( );   

    foreach my $rotfid (@rotfids) 
    {
	my @rots = split("@",$rotfid);
	
	unless (defined($rdfileidseen{$rotfid})) 
	{
	    # not seen, so a new rot id
	    # find only elements in @rotfids and not in @rdfilecontent
	    # element in @rotfids, but not in @rdfilecontent => This is a new rotfid
	    
	    # new visualization
	    push(@newrotids, $rotfid);
	}
	else
	{
	    my $line = $rdfileidseen{$rotfid};
	    push(@wrfilecontent,$line); 
	    
	    my @arline = split(",",$line);
	    my $href = $arline[2];
	    
	    if ($page =~ /$href/)
	    {
		#precreated visualization
		#print "precreated - $href\n";
		#push (@vizlinks,$href."/".$visualization_metalinks{$probeip});
		push (@vizlinks,$rots[0]."@".$href."/".$visualization_metalinks{$probeip});
	    }
	    else
	    {
		#stale link
		#print "stale:$href\n";
		#@wrfilecontent = grep {$_ !~ /$rotfid/} @wrfilecontent;
		#for (@wrfilecontent) 
		#{
		#    s/$rotfid//;	
		#}
		@wrfilecontent = grep {$_ !~ /$rots[1]/} @wrfilecontent;
		push(@stale_matched_rotfids,$rotfid);
	    }
	}
    }
    
    #print "\nNEW:::\n";
    #print @newrotids; 
    #print "\nStale Match:::\n";
    #print @stale_matched_rotfids;
    
    my @all_new_req_rotfids = (@newrotids,@stale_matched_rotfids) ; 
    
    #print "\nAll:::\n";
    #print @all_new_req_rotfids;
    
    if (@all_new_req_rotfids)
    {
	foreach my $all(@all_new_req_rotfids)
	{
	    #print "\nnon_matched_rotfids = $allrotfid\n";
	    my @allfile = split("@",$all) ; 
	    my  $allrotname = $allfile[0];
	    my  $allrotfid = $allfile[1];
	    
	    $req = HTTP::Request->new(GET => "http://$probeip/vision/visualizations/");
	    $req->method("POST");
	    $req->content_type("application/x-www-form-urlencoded");
	    $req->content("add_vis_name=$eventname&add_vis_ids=$allrotfid&vis_type_id=3&add_vis_id=&add_vis=1&vis_id=&form_submit=Add+visualization&panel_workspace_id=");					
	    $resp = $ua->request($req);
	    #push(@wrfilecontent,$probeip.",".$allrotfid.",".$resp->header( "Location" ));
	    push(@wrfilecontent,$probeip.",".$all.",".$resp->header( "Location" ));
	    push (@vizlinks,$allrotname."@".$resp->header( "Location" )."/".$visualization_metalinks{$probeip});
	}
    }
    
    #now logout silently
    $req = HTTP::Request->new(GET => "http://$probeip/vision/user/logout/");
    $req->method("POST");
    $req->content_type("application/x-www-form-urlencoded");
    $resp = $ua->request($req);
    
    
    if ( open (VISION,">","$viz_indexfile") )
    {
	if (flock(VISION, 6))
	{
	    foreach (@wrfilecontent)
	    {
		my $line = "$_";
		#print $line;
		next unless $line =~ /\S/;
		print VISION "$_\n"; # Print each entry in our array to the file
	    }
	}
	VISION->flush;
	close VISION;
    }
    #print @vizlinks;
}


sub endace_pivot_nomatch
{
    use List::Util 'first';  
    my $probeip = shift;
    
    if ( open (VISION,"$viz_indexfile") )
    {
	while (my $line = <VISION>)  
        {
	    next unless $line =~ /\S/;
	    chomp $line;
	    if ((my $pip, my $ruid, my $href) = split(",", $line))
	    {
		if ($pip !~ '#')
		{
		   #print "vision link $line";
		   if ($pip =~ $probeip)
		   {
			my @filename = split("@",$ruid) ; 
			my  $allrotname = $filename[0];
			my $match = first { /$allrotname/ } @vizlinks;
			unless (defined($match))
			{
			    push (@vizlinks,$allrotname."@".$href."/".$visualization_metalinks{$probeip});
			}
		   }
		}
	    }
	}
	close VISION;
    }
}

sub flowsearchurl 
{
    #Build the flowsearch URL corresponding to each Endace Probe in the Conf File	

    #open OUTFILE , '>>', "/tmp/sessiondebug" if $endace_debug;  # Test Debug, Must be disabled in Production
    print OUTFILE "\nFunction Name - flowsearchurl\n" if $endace_debug;

    my $event_ref = shift;
    my (%event) = %{$event_ref};
    #print OUTFILE %event,"\n" if $endace_debug;
 
    my $user_ref = shift;
    my (%user) =  %{$user_ref};
    #print  OUTFILE %user,"\n" if $endace_debug;

    my $host_ref = shift;
    my (@hosts) = @{$host_ref};
    
    my @baseurl=();
    
    #print @_;
    my $flow_direction = shift;
    #print $flow_direction;
    
    foreach $a (@hosts) 
    {
	my $url_string = "time=$event{'qtime'}&spanBefore=$event{'sBefore'}&spanAfter=$event{'sAfter'}&host1=$event{'sip'}&host2=$event{'dip'}&protocol=$event{'proto'}";
	my $vzurl_string = "?breakdown_type=app&breakdown_num_results=10&breakdown_filters=%7B%22i%5Bsip%5D%22%3A%22".$event{'sip'}."%2C".$event{'dip'}."%22%2C%22i%5Bdip%5D%22%3A%22".$event{'sip'}."%2C".$event{'dip'}."%22%7D&play=0";
	
	print  OUTFILE "\nflow_direction: $flow_direction\n" if $endace_debug;
	
        if ($api_version eq 'v0')
	{
	    if ($flow_direction == 0)
	    {
		my $url_string = "time=$event{'qtime'}&spanBefore=$event{'sBefore'}&spanAfter=$event{'sAfter'}&host1=$event{'sip'}&host2=$event{'dip'}&protocol=$event{'proto'}";
            }
	    else
	    {
		my $url_string = "time=$event{'qtime'}&spanBefore=$event{'sBefore'}&spanAfter=$event{'sAfter'}&host2=$event{'sip'}&host1=$event{'dip'}&protocol=$event{'proto'}";
                
	    }
	}
	if ($api_version eq 'v1')
	{
	    $vzurl_string = "?breakdown_type=app&breakdown_num_results=10&breakdown_filters=%7B%22i%5Bsip%5D%22%3A%22".$event{'sip'}."%2C".$event{'dip'}."%22%2C%22i%5Bdip%5D%22%3A%22".$event{'sip'}."%2C".$event{'dip'}."%22%2C%22i%5Bsport%5D%22%3A%22".$event{'spt'}."-".$event{'proto'}."%2C".$event{'dpt'}."-".$event{'proto'}."%22%2C%22i%5Bdport%5D%22%3A%22".$event{'spt'}."-".$event{'proto'}."%2C".$event{'dpt'}."-".$event{'proto'}."%22%7D&play=0";
	    
	    if ($flow_direction == 0)
	    {
		#forward direction as extracted from the event
		$url_string = "time=$event{'qtime'}&spanBefore=$event{'sBefore'}&spanAfter=$event{'sAfter'}&host1=$event{'sip'}&host2=$event{'dip'}&port1=$event{'spt'}&port2=$event{'dpt'}&protocol=$event{'proto'}";
	    }
	    else
	    {
		#reverse direction than extracted from the event
		$url_string = "time=$event{'qtime'}&spanBefore=$event{'sBefore'}&spanAfter=$event{'sAfter'}&host1=$event{'dip'}&host2=$event{'sip'}&port1=$event{'dpt'}&port2=$event{'spt'}&protocol=$event{'proto'}";
	    }
	}

        print  OUTFILE "flowsearch url string ".$url_string."\n" if $endace_debug;
        push (@baseurl, "http://$a/api/$api_version/flowsearch?$url_string"); #base urls
        
        my $xmin = $event{'qtime'}-$event{'sBefore'};
	$xmin = $xmin."000";
	my $xmax = $event{'qtime'}+$event{'sAfter'};
	$xmax = $xmax."000";
	        
        #$visualization_links{$a} = "https://$a/vision/visualizations/view/$vzid/$vzurl_string"."&x_min=".$xmin."&x_max=".$xmax;
        #$visualization_links{$a} = "http://".$user{'login'}.":".$user{'pass'}."@".$a."/vision/visualizations/view/".$vzid."/".$vzurl_string."."."&x_min=".$xmin."&x_max=".$xmax;
        #print $visualization_links{$a};
        $visualization_metalinks{$a} = $vzurl_string."."."&x_min=".$xmin."&x_max=".$xmax;
    }
    #print OUTFILE @baseurl,"\n" if $endace_debug;
    #close OUTFILE if $endace_debug;

    return (@baseurl);
}


sub getendacelinks 
{
    my $rest_debug = 0; #local Restdebug - Non Production 

    my $event_ref = shift;
    my (%event) = %{$event_ref};

    my $user_ref = shift;
    my (%user) =  %{$user_ref};

    my $host_ref = shift;
    my (@hosts) = @{$host_ref};
    
    my $flow_direction = shift;
    
    my @baseurl=();

    my @downloadurls = ();

    my %flowbytecount = ();
    
    if ($rest_debug == 1 and $endace_debug == 1)
    {
	open RESTDBG , '>', "rest_debug" if $rest_debug;  # Test Debug, Must be disabled in Production
    }
    print OUTFILE "\nFunction Name - getendacelinks\n" if $endace_debug;

    #build flowsearch urls here
    # New : The last parameter 0 [forward], 1[reverse] is the flow direction
    
    my @flowurls = flowsearchurl (\%event,\%user,\@hosts, $flow_direction);
    print OUTFILE @flowurls,"\n" if $endace_debug;	

    #foreach created flowsearch url, do the following - 
    # 1. GET - Flowsearch URL
    # 2. Parse returned JSON datastructure & Extract fields (rotationfiles & hrefs) to create datamine URLs - only if flowsearch returns valid data
    # if 2 returns valid data - then proceed to 3,4,5	
    # 3. POST - datamine -this will create a datamine session on the Endace Probe
    # 4. Parse Return JSON datastructure & Extract exact Datamine URLs
    # 5. If Debug enabled - DELETE all datamine sessions as and when they are created - Later need to open a clean up function to do the same

    foreach $a (@flowurls) 
    {
        my @rotuids = ();
        
        print RESTDBG "$a" if $rest_debug;
        
	#foreach flowsearch matches, extract the datamine urls
	#/([0-9a-zA-Z\_\-\.]+)/
	
	my @probeip = split('/', $a);
	#print "\nProbe = $probeip[2]"."\n";
	
	#set browser agent
        my $ua = LWP::UserAgent->new(); 
        $ua->agent("Mozilla/5.0 (Windows NT 6.1; WOW64; rv:15.0) Gecko/20120427 Firefox/15.0a1");
	$ua->cookie_jar( {} );
  
        # set custom HTTP request header fields
        my $req = HTTP::Request->new(GET => $a);
        $req->authorization_basic($user{'login'},$user{'pass'});
        $req->header('content-type' => 'application/json');

	#initiate HTTP request
        my $resp = $ua->request($req);
        
        #process HTTP request on success code 200 ok
        if ($resp->is_success) 
        {
            my $message = $resp->decoded_content;
            my $json_msg = decode_json($message);
            print RESTDBG Dumper $json_msg,"\n" if $rest_debug;

	    #check the flowsearch results to determine if the query matched anything or returned zero bytes match
	    # if zero bytes match, exit this function and inform the calling API that no data found.

	    my %total = %{$json_msg->{'results'}{'total'}};
	    my @datamineurls = (); 

           if ($endace_debug) 
	    {
	        print "Total Flowsearch ByteCount $total{'flowByteCount'}","\n" if $endace_debug_print;
	    }

	    
	    if ($total{"totalByteCount"} > 64)
	    {
                my @rots = @{$json_msg->{'results'}{'rotationFiles'}};
                foreach my $rotationlinks (@rots) 
	        {
                    print RESTDBG $rotationlinks->{"rotationFileName"},"\n" if $rest_debug;
		    #print "<br><dd>- Found <b>$total{'flowByteCount'} Bytes </b> from RotationFile <b>$rotationlinks->{'rotationFileName'} </b></br></dd>","\n"; 	    	
		    #push (@rotuids,$rotationlinks->{'rotationFileID'});
		    push (@rotuids,$rotationlinks->{"rotationFileName"}."@".$rotationlinks->{'rotationFileID'});
		    
                    my @links = @{$rotationlinks->{"links"}};
                    foreach my $link (@links) 
		    {
                        my @params = @{$link->{"fields"}};
                        my $link = $link->{"href"};
                        my %data = ();
                        foreach my $var (@params) 
		        {
                            #print $var->{'name'}."\n"; 
                            #print $var->{'value'}."\n"; 
                            $data{$var->{'name'}} = $var->{'value'}; 
                        }
    		        my $url_string = "$link?rotationFiles=$data{'rotationFiles'}&startTime=$data{'startTime'}&endTime=$data{'endTime'}&filterTCPDump=$data{'filterTCPDump'}";
			print $url_string,"\n" if $endace_debug_print;
    		        push (@datamineurls,$url_string);
			#adding the flowbyte count into a hash , to retreive it when valid download url is vailable
                        $flowbytecount{$url_string}="<b>$rotationlinks->{'flowByteCount'}</b>"." Bytes and <b>$rotationlinks->{'flowPacketCount'}</b>"." Packets found from RotationFile <b>$rotationlinks->{'rotationFileName'}</b> on Endace Probe <b>$rotationlinks->{'probe'}</b>";

    		    }
    	        }
		generate_viz_link(\%user,\@rotuids, $probeip[2],"SourceFireEvents_From_DE_ID_".$event{'sfdcid'});
	    }
	    #generate_viz_link(\%user,\@rotuids, $probeip[2],"SourceFireEvents_From_DE_ID_".$event{'sfdcid'});
  
	    #foreach dataminedurls created by each matching flowsearch sessions, do a POST to create datamine
	    foreach my $mineurl (@datamineurls) 
	    {
		print RESTDBG $mineurl, "\n" if $rest_debug;
		my $pua = LWP::UserAgent->new(); 
		# set custom HTTP request header fields for POST
	        my $preq = HTTP::Request->new(POST => $mineurl);
		$preq->authorization_basic($user{'login'},$user{'pass'});
		my $presp = $pua->request($preq);
		if ($presp->is_success) 
        	{
		    #for every successful creation of datamine URL, store the links in an array to return back to calling function.	
		    my $datamine_message = $presp->decoded_content;
                    my $pjson_msg = decode_json($datamine_message);
            	    print RESTDBG Dumper $pjson_msg,"\n" if $rest_debug;

		    my @datamine_message = @{$pjson_msg->{'messages'}};

		    print RESTDBG $datamine_message[0]->{'description'},"\n" if $rest_debug;
    		    print RESTDBG $datamine_message[0]->{'name'},"\n" if $rest_debug;
		    print RESTDBG $datamine_message[0]->{'type'},"\n" if $rest_debug;
		
		    my @dataminelinks = @{$pjson_msg->{'links'}};
		    my @deldatamines = ();
		    
		    if ($datamine_message[0]->{'type'} eq 'success')
		    {
		        foreach my $dlinks (@dataminelinks) 
		        {
		            if ($dlinks->{"name"} eq "downloadPcap")
			    {
			        print RESTDBG $dlinks->{"href"},"\n" if $rest_debug;
				
				my $uad = LWP::UserAgent->new();
				$uad->max_size(64);	
			        my $reqd = HTTP::Request->new(GET => $dlinks->{"href"});
			        $reqd->authorization_basic($user{'login'},$user{'pass'});
				my $valid_content_length_flag = 0;
				my $resd = $uad->request($reqd,
				        	sub 
				        	{
				            	    my($chunk,$resd) = @_;
						    #if ($resd->content_length > 0)	
						    if (length($chunk) > 0)
						    {
							#print $resd->content_length;
							$valid_content_length_flag = 1 ;
							return;
						    }
						});
				if ($valid_content_length_flag)
			        {
				    my $printable_string = "Download PCAP of size $flowbytecount{$mineurl}";
				    push(@downloadurls,$printable_string."@".$dlinks->{"href"});
				    #push(@downloadurls,$dlinks->{"href"});
				    #print "<br><dd>- Estimating Packet Data of size $flowbytecount{$mineurl}</br></dd>","\n";
				}
			        else
				{
				    print "<br><dd>- Packet Data has rotated out of Endace Probe </br></dd>","\n";
				}	
				#push(@downloadurls,$dlinks->{"href"});
			    
	                        if ($dlinks->{"name"} eq "deleteDatamine")
			        {
			            print RESTDBG $dlinks->{"href"},"\n" if $rest_debug;
			            push(@deldatamines,$dlinks->{"href"});
			        }
			        #now logout silently
				$reqd = HTTP::Request->new(GET => "http://$probeip[2]/vision/user/logout/");
				$reqd->method("POST");
				$reqd->content_type("application/x-www-form-urlencoded");
				$resp = $uad->request($reqd);
			    }			
		    	}
			
		    }
		    
		}
		else
		{
		    print RESTDBG "\nDatamine - HTTP GET error code: ", $resp->code, "\n" if $rest_debug;
		    print RESTDBG "\nDatamine - HTTP GET error message: ", $resp->message, "\n" if $rest_debug;
		    my $code = $resp->code;
		    my $mesg = $resp->message; 	
		    # print the code and mesg to let the user know the returned error code : TBD
		    print "<br><dd>- Datamine Unsuccessful - Check RBAC roles on Endace Probes</dd></br>";
		    	
		}
		
	    }
   
	}
        else 
	{
	    print RESTDBG "\nFlowSearch - HTTP GET error code: ", $resp->code, "\n" if $rest_debug;
	    print RESTDBG "\nFlowSearch - HTTP GET error message: ", $resp->message, "\n" if $rest_debug;
            my $code = $resp->code;
	    my $mesg = $resp->message; 		
	    # print the code and mesg to let the user know the returned error code : TBD
	    endace_pivot_nomatch($probeip[2]);
	    if ($flow_direction == 0)
	    {
		print "<br><dd>- Flowsearch Unsuccessful between host a->b - May be due to invalid layer 4 flow or permissions on Endace Probes </dd></br>";
	    }
	    else
	    {
		print "<br><dd>- Flowsearch Unsuccessful between host b->a - May be due to invalid layer 4 flow or permissions on Endace Probes </dd></br>";
	    }
	    
	}
	#now logout silently
	$req = HTTP::Request->new(GET => "http://$probeip[2]/vision/user/logout/");
	$req->method("POST");
	$req->content_type("application/x-www-form-urlencoded");
	$resp = $ua->request($req);

    
    }
    #close OUTFILE if $endace_debug;
    close RESTDBG if $endace_debug;

    return @downloadurls;
}

sub mkRequest
{
    my $flow_direction = 0;
    my ($sfevent) = @_;
    #open OUTFILE , '>', "/tmp/sessiondebug" if $endace_debug;  # Test Debug, Must be disabled in Production

    #For each entry in the conf file, search for flows and if found create PCAP URLs to download data from them
    
    my @global_urls=(); # list of URLs that matches across Endace Probes for a specific event

    #open OUTFILE , '>>', "/tmp/sessiondebug" if $endace_debug;  # Test Debug, Must be disabled in Production
    print OUTFILE "\nFunction Name - mkRequest\n" if $endace_debug;

    #parse event messages to extract the variables to construct a href
    my $session = getsession($sfevent); 
    
    #read the endace configfile 		    	
    my $configparam = readconfigfile($session->{'sensor'});

    if ( $endace_integration_enable )
    {
	for my $de ( keys %$configparam ) 
	{
	    my %event=();
	    my %user=();
	    my @hosts=();

	    if ($endace_debug)
	    {
		print OUTFILE "\nprinting conf params of $de\n";
		print OUTFILE $configparam->{$de}->{'epAddr'}.",\n";
		print OUTFILE $configparam->{$de}->{'epUser'}.",\n";
		print OUTFILE $configparam->{$de}->{'epPass'}.",\n";
		print OUTFILE $configparam->{$de}->{'epsBefore'}.",\n";
		print OUTFILE $configparam->{$de}->{'epsAfter'}.",\n";
        
	    }

	    #create & fill endace specific datastructures
	    # 1. event related

	    $event{'sfdcid'} = $de;
	    #print "<br><b>Sourcefire DE in Configuration File:</b> $de </br> " ;

	    $event{'qtime'} = $session->{'timestamp'};
	    print "<br><dd>- timestamp = $session->{'timestamp'} </dd></br>" if $endace_debug_print;

	    $event{'sBefore'} = $configparam->{$de}->{'epsBefore'};
	    print "<br><dd>- epsBefore = $configparam->{$de}->{'epsBefore'} </dd></br>" if $endace_debug_print;

	    $event{'sAfter'} = $configparam->{$de}->{'epsAfter'};
	    print "<br><dd>- epsAfter = $configparam->{$de}->{'epsAfter'} </dd></br>" if $endace_debug_print;

	    $event{'sip'} = $session->{'sip'};
	    print "<br><dd>- sip = $session->{'sip'} </dd></br>" if $endace_debug_print;

	    $event{'dip'} = $session->{'dip'};
	    print "<br><dd>- dip = $session->{'dip'} </dd></br>" if $endace_debug_print;

	    $event{'spt'} = $session->{'spt'};
	    print "<br><dd>- spt = $session->{'spt'} </dd></br>" if $endace_debug_print;

	    $event{'dpt'} = $session->{'dpt'};
	    print "<br><dd>- dpt = $session->{'dpt'} </dd></br>" if $endace_debug_print;

	    $event{'proto'} = $session->{'protocolname'};
	    print "<br><dd>- protocolname = $session->{'protocolname'} </dd></br>" if $endace_debug_print;
	
	    # 2. User credentials
	    $user{'login'} = $configparam->{$de}->{'epUser'} ;
	    print "<br><dd>- epUser = $configparam->{$de}->{'epUser'} </dd></br>" if $endace_debug_print;

	    $user{'pass'} = $configparam->{$de}->{'epPass'};
	    print "<br><dd>- epPass = $configparam->{$de}->{'epPass'} </dd></br>" if $endace_debug_print;

	    # 3. Endace Probe IP Address - 1 Probe now at a time, right now this is an array, will change to a string variable later
	    push (@hosts,$configparam->{$de}->{'epAddr'});
	    print "<b>Endace Host:</b> <td>$configparam->{$de}->{'epAddr'} </td>","\n";

	    # 4. push the return URLs in global url array for calling function to process
	    my @eplinks = getendacelinks(\%event,\%user,\@hosts, $flow_direction);
	    if (@eplinks)
	    {
		push (@global_urls,@eplinks);
	    }
	    else
	    {
		$flow_direction = 1;
		my @eplinks = getendacelinks(\%event,\%user,\@hosts, $flow_direction);
		if (@eplinks)
		{
		    push (@global_urls,@eplinks);
		}
		else
		{
		    print "<br><dd>- No Packet Data found from Endace Host: $configparam->{$de}->{'epAddr'} </br></dd>","\n";
		}
	    }
	}
	if ($endace_debug)
	{	
	    foreach my $gurl (@global_urls)
	    {
		print OUTFILE $gurl,"\n";
	    }
	}
    }
    close OUTFILE if $endace_debug;
    return @global_urls;
}

#Endace_Integration_Patch

# This function is used by the automation team for product testing.
# If you change the arguments, output, or functionality of this function
# please open a bug:
#
# Product: Automation
# Component: ATF:Framework
# Subject should start with: SF_API Changes
#
# Quick link below:
# https://bugzilla.sourcefire.com/enter_bug.cgi?product=Automation&component=ATF:Framework
sub getPacketInformationForEvent
{
    my ( $query, $eventInfo, $options ) = @_; #, $eventData
    warn " ** SF::PacketView::getPacketInformationForEvent\n" if ( $_debug );
    warn Dumper( $eventInfo ) if ( $_debug && $_debug > 1 );

    my $gensid = $eventInfo->{gidsid};
    my $adddstinfo = 0;
    if ( $eventInfo->{gidsid} && $eventInfo->{gen} && $eventInfo->{sid} )
    {
        $adddstinfo = 1 if (($eventInfo->{gen} == 122) &&
            (( $eventInfo->{sid} == 3 ) || ( $eventInfo->{sid} == 7 )));
    }
    my $sandBox = undef;
    if ( $_debug && $_debug > 2 )
    {
        $sandBox = tempdir('PacketView_XXXX',
            DIR => SF::Reloc::RelocateFilename('/var/tmp'));
        warn "relocated sandbox $sandBox\n";
    }
    else
    {
        $sandBox = tempdir('PacketView_XXXX',
            DIR => SF::Reloc::RelocateFilename('/var/tmp'), CLEANUP => 1);
    }
    $sandBox = SF::Reloc::UnRelocateFilename( $sandBox );
    warn "    Sandbox location $sandBox\n" if ( $_debug );

    my $var;
    $var->{cgiSF} = $query;
    $var->{sandBox} = $sandBox;
    $var->{pcapPath} = $sandBox . "/pcap";
    $var->{addDSTInfo} = $adddstinfo;
    $var->{outputType} = "HTML" unless($options);
    $var->{outputType} = "PDF_REPORT" if ($options && $options->{pdf});
    $var->{outputType} = "HTML_REPORT" if ($options && $options->{html});
    $var->{eventCount} = ($options && $options->{eventCount})? $options->{eventCount} : 1;
    $var->{username} = (defined $options && defined $options->{username} && $options->{username} ne '') ? $options->{username} : undef;
    my $pcap_generated = SF::PacketRender::EventPacketsToPcap({sensor_id => $eventInfo->{sensor},
                                                      event_id  => $eventInfo->{event_id},
                                                      event_sec => $eventInfo->{tvsec},
                                                      pcap_file => $var->{pcapPath}});
    warn "    PCAP export status : $pcap_generated\n" if ( $_debug );
    #
    # This is the filter we pass to tethereal for cmg's special packets
    #
#    my $packet_info_filter_expression = "(eth.src==4D:41:43:44:41:44 && eth.dst == 4D:41:43:44:41:44 && eth.type==0x0800 && ip.ttl==0)";
#    system("/usr/bin/tethereal -Vnxr $sandBox/pcap \"$packet_info_filter_expression\" > $sandBox/tethereal.out");

    my $retString;
    try
    {
        my $data = SF::System::tshark( tempDir => $sandBox, decode_type => "port_scan" ) if($pcap_generated);
        warn Dumper( $data ) if ( $_debug && $_debug > 2 );

        if ( $data->{stdout} )
        {
            if ( $var->{outputType} =~ /HTML/ )
            {
            my @lines = split( "\n", $data->{stdout} );
            $var->{port_scan_data} = \@lines;
            $var->{packet_data} = undef;
            processPortscan( $var, $eventInfo );
            $retString = join "\n", @{$var->{display_list}};
                $retString = "<table class=simple-table>"
                    . $retString . "</table>";
            }
            else
            {
                $retString = $data->{stdout};
            }
        }
        else
        {
            $data = SF::System::tshark( tempDir => $sandBox, decode_type => "packet" ) if($pcap_generated);
            while ( $data->{stdout} =~ /\\[nr]$/m )
            {
                $data->{stdout} =~ s/\\[nr]$//gm;
            }
            warn Dumper( $data ) if ( $_debug && $_debug > 2 );
            my @lines = split( "\n", $data->{stdout} );
            if ( $var->{outputType} =~ /HTML/ )
            {
                $var->{packet_data} = \@lines;
                $var->{port_scan_data} = undef;
                $var->{showAll} = 1;
                #$var->{eventData} = $eventData;
                processPacket( $var, $eventInfo );
            }
            else
            {
                $var->{display_string}  = draw_event( $var, $eventInfo );
                $var->{display_string} .= "Packet Information\n";
                $var->{display_string} .= $data->{stdout};
            }
            $retString = $var->{display_string};
        }
    }
    catch Error with
    {
        my $E = shift;
        warn Dumper( $E ) if ( $_debug );
        warn Dumper( $E );
        warn "Caught Exception ** while decoding packet - IGNORE\n";
        $retString = "";
    };

    warn "getPacketInformationForEvent - rendering text\n$retString\n" if ( $_debug );
    return $retString;
}

sub initalJavascript
{
    my ($var) = @_;

    return ""  if ( $var->{eventCount} != 1);

    my $expand_all = SF::i18n::get_pm_msg( "expand_all", "PacketView");
    my $hide_all = SF::i18n::get_pm_msg( "hide_all", "PacketView");

    my $scriptText =<<END;
<style>
    pre {
        margin    : 0;
        font-size : 8pt !important;
    }

    .section {
        margin-bottom : 8px;
    }

    .section h2 {
        display       : inline-block;
        padding-right : 12px;
        margin        : 4px -12px 4px 0px;
    }

    .section h2 b {
        font-weight : normal;
    }

    .section th {
        width        : 160px;
        padding-left : 0px !important;
    }

    .side-bar .section th {
        width : 200px;
    }

    .empty.section h2 {
        background : none;
        cursor     : auto;
    }
</style>
END

    if ($var->{outputType} eq 'HTML') {
        $scriptText.=<<END;
<style>
    .section h2 {
        cursor        : pointer;
        background    : url(/img/triangle-down-red.png) 0 4px no-repeat;
        background-position : right;
    }

    .side-bar .section .content {
        margin-left : 12px;
    }

    .side-bar .section h3 {
        display      : inline-block;
        padding-left : 12px;
        margin       : 0px 0px 0px 0px;
    }

    .side-bar .section h3 {
        cursor       : pointer;
        background   : url(/img/triangle-down.png) 0 4px no-repeat;
    }

    .collapsed.section .content, .empty.section .content {
        display : none;
    }

    .collapsed.section h2 {
        background-image : url(/img/triangle-right-red.png);
    }

    .collapsed.section h3 {
        background-image : url(/img/triangle-right.png);
    }

    .frame-header {
        background : #dce6f4;
        margin     : -5px -5px 8px;
        padding    : 4px 17px;
    }
</style>

<script langauge=Javascript>
var maxSections = 0;

function toggleSection(id) {
    var section = document.getElementById(id);
    section.className = section.className === "collapsed section"
                      ? "section"
                      : "collapsed section";
}

function toggle( img )
{
    var section = document.getElementById(img.id.replace(/img/, "section"));

    if( img.src.match( "triangle-down.png" ) )
    {
        img.src = "$imgpath/triangle-right.png";
        section.style.display = "none";
    }
    else
    {
        img.src = "$imgpath/triangle-down.png";
        section.style.display = "";
    }
}

function initColumns()
{
    var divs = document.getElementsByTagName("div");
    for (var i = 0; i < divs.length; i++)
    {
        if (divs[i].className.indexOf("collapsible") > -1)
        {
            var img   = document.getElementById(divs[i].id.replace(/section/, "img"));
            var label = document.getElementById(divs[i].id.replace(/section/, "label"));
            if (img && label && divs[i].offsetHeight > label.offsetHeight)
            {
                divs[i].style.height = label.offsetHeight;
                img.style.display = "";
            }
        }
    }
}

function toggleColumn( img )
{
    var label   = document.getElementById(img.id.replace(/img/, "label"));
    var section = document.getElementById(img.id.replace(/img/, "section"));

    if( img.src.match( "triangle-down.png" ) )
    {
        img.src = "$imgpath/triangle-right.png";
        section.style.height = label.offsetHeight;
    }
    else
    {
        img.src = "$imgpath/triangle-down.png";
        section.style.height = "auto";
    }
}

function expandFrame( a, id )
{
    var tState;
    if( a.innerHTML == "$expand_all" )
    {
        tState = "section";
        a.innerHTML = "$hide_all";
    }
    else
    {
        tState = "collapsed section";
        a.innerHTML = "$expand_all";
    }
    var idx = 1;
    var tObj;
    while (tObj = document.getElementById( "section_" + id + "_" + idx )) {
        tObj.className = tState;
        idx++;
    }
}

function changePacketDataColour( id, type )
{
    if (type)
    {
        document.getElementById("HEX_" + id).style.backgroundColor = "yellow";
        document.getElementById("ASC_" + id).style.backgroundColor = "yellow";
    }
    else
    {
        document.getElementById("HEX_" + id).style.backgroundColor = null;
        document.getElementById("ASC_" + id).style.backgroundColor = null;
    }
}

var sectionID = new Array();
var userOption = $expandState;
function expandSection( str )
{
    sectionID[sectionID.length] = str;
}

function expandOnLoad()
{
    for (var i = 0; i < sectionID.length; i++)
    {
        var obj = document.getElementById( sectionID[i] );
        if ( obj && userOption && userOption != 2 ) { toggle( obj ); }
        if ( obj && userOption == 2 ) { expandFrame( obj, sectionID[i] ); }
    }
}
  // This routine handles hiding/un-hiding the threshold &
  // suppression stuff.
  //
  // If we open one, we close all of the others.
  //
  function toggleThreshSuppBlocks(blockId)
  {
    var block = document.getElementById(blockId);
    var blockStatus = block.style.display;

    if(blockId == "thresh_current" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_all");
       hideThreshSuppBlock("suppress_current");
       hideThreshSuppBlock("suppress_all");
       showThreshSuppBlock("thresh_current");
    }
    else if(blockId == "thresh_current" && blockStatus == "block")
    {
       hideThreshSuppBlock("thresh_current");
    }
    else if(blockId == "thresh_all" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_current");
       hideThreshSuppBlock("suppress_current");
       hideThreshSuppBlock("suppress_all");
       showThreshSuppBlock("thresh_all");
    }
    else if(blockId == "thresh_all" && blockStatus == "block")
    {
       hideThreshSuppBlock("thresh_all");
    }
    else if(blockId == "suppress_current" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_current");
       hideThreshSuppBlock("thresh_all");
       hideThreshSuppBlock("suppress_all");
       showThreshSuppBlock("suppress_current");
    }
    else if(blockId == "suppress_current" && blockStatus == "block")
    {
       hideThreshSuppBlock("suppress_current");
    }
    else if(blockId == "suppress_all" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_current");
       hideThreshSuppBlock("thresh_all");
       hideThreshSuppBlock("suppress_current");
       showThreshSuppBlock("suppress_all");
    }
    else if(blockId == "suppress_all" && blockStatus == "block")
    {
       hideThreshSuppBlock("suppress_all");
    }
    else if(blockId == "suppress_current1" )
    {
    hideThreshSuppBlock("suppress_current2");
    hideThreshSuppBlock("suppress_current3");
    showThreshSuppBlock("suppress_current1");
    nameswap1("suppress_current1","suppress_current2","suppress_current3");
    }
    else if(blockId == "suppress_current2" )
    {
    hideThreshSuppBlock("suppress_current1");
    hideThreshSuppBlock("suppress_current3");
    showThreshSuppBlock("suppress_current2");
    nameswap1("suppress_current2","suppress_current1","suppress_current3");
    }
    else if(blockId == "suppress_current3" )
    {

    hideThreshSuppBlock("suppress_current1");
    hideThreshSuppBlock("suppress_current2");
    showThreshSuppBlock("suppress_current3");
    nameswap1("suppress_current3","suppress_current2","suppress_current1");
    }
    else if(blockId == "suppress_all1" && blockStatus != "block")
    {
    hideThreshSuppBlock("suppress_all2");
    hideThreshSuppBlock("suppress_all3");
    showThreshSuppBlock("suppress_all1");
    nameswap2("suppress_all1","suppress_all2","suppress_all3");
    }
    else if(blockId == "suppress_all2" && blockStatus != "block")
    {
    hideThreshSuppBlock("suppress_all1");
    hideThreshSuppBlock("suppress_all3");
    showThreshSuppBlock("suppress_all2");
    nameswap2("suppress_all2","suppress_all3","suppress_all1");
    }
    else if(blockId == "suppress_all3" && blockStatus != "block")
    {
    hideThreshSuppBlock("suppress_all2");
    hideThreshSuppBlock("suppress_all1");
    showThreshSuppBlock("suppress_all3");
    nameswap2("suppress_all3","suppress_all2","suppress_all1");
    }

   }
  function nameswap2(blockId1, blockId2, blockId3)
  {
     block1=document.getElementById(blockId1);
     block2=document.getElementById(blockId2);
     block3=document.getElementById(blockId3);
     block1.name="suppress_all_address"
     block2.name="suppress_not_all1"
     block3.name="suppress_not_all2"
  }
  function nameswap1(blockId1, blockId2, blockId3)
  {
     block1=document.getElementById(blockId1);
     block2=document.getElementById(blockId2);
     block3=document.getElementById(blockId3);
     block1.name="suppress_current_address"
     block2.name="suppress_not_current1"
     block3.name="suppress_not_current2"
  }
  //This shows a T & S block
  function showThreshSuppBlock(blockId)
  {

    // errors in here are OK.  catch and ignore them
    try
    {
       block = document.getElementById(blockId);
       blockImg = document.getElementById(blockId + "_img");

       block.style.display = "block";
       blockImg.src = "$imgpath/triangle-down.png";
    }
    catch (e)
    {
    //dbg_msg(e);
    }
  }


  // this hides a T & S block
  function hideThreshSuppBlock(blockId)
  {
    // Errors in here are OK. catch and ignore them
    try
    {
      block = document.getElementById(blockId);
      blockImg = document.getElementById(blockId + "_img");

      block.style.display = "none";
      blockImg.src = "$imgpath/triangle-right.png";
    }
    catch (e)
    {
      //dbg_msg(e);
    }
  }
END

        $scriptText .= "addOnLoad( initColumns ); addOnLoad( expandOnLoad );" if ( $var->{outputType} eq "HTML" );
        $scriptText .= "</script>";
    }

    return $scriptText;
}

sub processPacket
{
    my ( $var, $eventInfo ) = @_;

    my $timeString = SF::Render::render( "event_time", $eventInfo->{tvsec} );
    my $eventHtml  = draw_event( $var, $eventInfo );
    my $scriptText = initalJavascript($var);

    my $html = $scriptText.$eventHtml;

    if ( !scalar(@{$var->{packet_data}} ))
    {
        $html .= '<h2>'. SF::i18n::get_pm_msg("No Packet Information", "PacketView") .'</h2>';
        $var->{display_string} = $html;
        return 1;
    }
    $html .= '<h2>' . SF::i18n::get_pm_msg( "Packet Information", "PacketView") . '</h2>' if ( $var->{outputType} eq "HTML" );
    my $offset = 0;
    my $payload = [];
    my $data = [];
    my $numLines = scalar( @{$var->{packet_data}} );
    my $payloadMode = 0;
#
#FRAME 1 (610 bytes on wire, 610 bytes captured)
#    Arrival Time: May 29, 2008 21:27:12.825089000
#...
#0000  00 20 a6 4c 47 15 08 00 20 f1 2f 35 08 00 45 00   . .LG... ./5..E.
#
#                 --OR--
#
#FRAME 1 (610 bytes on wire, 610 bytes captured)
#    Arrival Time: May 29, 2008 21:27:12.825089000
#...
#
#Frame (610 bytes):
#
#0000  00 20 a6 4c 47 15 08 00 20 f1 2f 35 08 00 45 00   . .LG... ./5..E.
#...
#
#De-chunked entity body (290 bytes):
#
#0000  3c 21 44 4f 43 54 59 50 45 20 48 54 4d 4c 20 50   <!DOCTYPE HTML P
#...
#
#
#Definition of "Packet Byte "
# 1. No Title line and Lines Starting with number
# 2. Title line, any-number of blank lines and Lines Stating with number
# 3. Any number of type 2 format with blank line(s) in between.
#
#Note: Once Packet Byte starts, it goes till the end or start of next FRAME.
#
    for( my $pos = 0; $pos < $numLines; $pos++ )
    {
        my $line = $var->{packet_data}[$pos];
        next if( $line =~ /^\s*$/ );
        if ( $line =~ /^Frame.*bytes on wire.*bytes captured/ )
        {
            $html .= draw_packet({ showAll => $var->{showAll},
                data => $data,
                payload => $payload,
                eventCount => $var->{eventCount},
                dc => $eventInfo->{sensor},
                }, $var) if (scalar(@{$payload}) && scalar(@{$data}));
            $payload = [];
            $data = [ $line ];
            $offset = 0;
            $payloadMode = 0;
        }
        elsif ($payloadMode)
        {
            push @{$payload}, $line;
        }
        else
        {
            next if ( !$var->{showAll} );
            $line ="$1$2$3" if ( $line =~ /^(\s*)\[(.*)\](\s*)$/ );
            if ( $line =~/^\w/ )
            {
                my $loc = $pos+1;
                while( length($var->{packet_data}[$loc]) == 0 )
                {
                    $loc++;
                }
                if ( $var->{packet_data}[$loc] =~ /^\d+/ )
                {
                    $payloadMode = 1;
                    push @{$payload}, $line;
                    next;
                }
            }
            push @{$data}, $line;
        }
    }
    $html .= draw_packet({ showAll => $var->{showAll},
        data => $data,
        payload => $payload,
        eventCount => $var->{eventCount},
        dc => $eventInfo->{sensor},
        }, $var);
    $var->{display_string} = $html;
    return 1;
}

sub draw_event
{
    my ( $var, $event ) = @_;

    my $key        = "$var->{eventCount}_0_0";
    my $subkey     = 0;
    my $timeString = SF::Render::render( "event_time", $event->{tvsec} );

    my $columns = { map { $_->{'name'} => $_ } @{$event->{'columns'}} };

    # render all columns
    foreach my $column (@{$event->{'columns'}})
    {
        # use packetview IP renderer
        if ($column->{'type'} eq 'ip')
        {
            $column->{'type'} = 'packetview.ip';
        }

        # do not truncate column data
        elsif ($column->{'type'} eq 'long_string')
        {
            $column->{'type'} = 'string';
        }

        if (defined $column->{'data'} && $column->{'data'} =~ /:/)
        {
            my @render_args = ($column->{'type'}, $column->{'data'});
            $column->{'display_data'} = SF::Render::renderExtraText(@render_args) . SF::Render::render(@render_args);
        }
        else
        {
            $column->{'display_data'} = SF::Render::render($column->{'type'}, $column->{'data'});
        }

        if (!defined $column->{'display_data'} || $column->{'display_data'} =~ /^\s*$/)
        {
            $column->{'display'} = 0 unless $column->{'display'};
            $column->{'display_data'} = '<span style="color: gray;">' . SF::i18n::get_pm_msg('no_data', 'PacketView') . '</span>';
        }
        else
        {
            $column->{'display'} = 1;
            if (defined $column->{'dependencies'})
            {
                foreach my $dependency (grep { exists $columns->{$_} } @{$column->{'dependencies'}})
                {
                    $columns->{$dependency}{'display'} = 1;
                }
            }
        }
    }

    my $retString;

    if ($var->{'outputType'} eq "PDF_REPORT")
    {
        $retString = "Event Information\nEvent: $event->{display_text}, Timestamp: $timeString\n";
        foreach my $column (grep { $_->{'display'} } @{$event->{'columns'}})
        {
            $retString .= $column->{'display_name'} . ' : ' . $column->{'display_data'} . "\n";
        }
        return $retString;
    }

    my ($rule_obj, $ruleText);
    try
    {
        my $idsEventMsgMap    = SF::IDS::Events::Map::getMsgEntryBySID($event->{'sid'}, $event->{'gen'});
        $event->{'rule_uuid'} = $idsEventMsgMap->{'uuid'};
        $event->{'rule_rev'}  = $idsEventMsgMap->{'rev_uuid'};
        $rule_obj             = SF::EOHandler::loadObject($event->{'rule_uuid'});
    }
    catch Error with
    {
        $rule_obj = undef;
    };
    if ($event->{'gen'} != 138) # bug 78069 - hide rule text for GID 138 events
    {
        try
        {
            my $rule_rev    = SF::EOHandler::loadObject($event->{'rule_uuid'}, $event->{'rule_rev'});
            $event->{'sid'} = $rule_rev->{'data'}{'sid'};
            $event->{'gen'} = $rule_rev->{'data'}{'gid'};
            my $username = (defined $var and exists $var->{username} and defined $var->{username} and $var->{username} ne '') ? $var->{username} : undef;
            if ($event->{'sid'} >= 1000000 and !SF::Permission::hasAnyPermission(["local_rules","rules"], $username)) {
                $ruleText = "Rule text view disabled for sid: $event->{'sid'}";
#                $ruleText = "Rule text view disabled for sid: $event->{'sid'}<pre>var is ".Dumper($var)."\nevent is ".Dumper($event)."\nstacktrace is ".SF::Util::Stacktrace::ToString()."</pre>";
            } else {
                $ruleText = HTML::Entities::encode(SF::Intrusion::Rule::Parse::gen_rule($rule_rev->{'data'}));
            }
            unless ($rule_obj)
            {
                my $owner = SF::PeerManager::Peers::getPeerInfo($rule_rev->{'authority'});
                $ruleText .= '&nbsp;<i>' . SF::i18n::get_format_pm_msg("Remote_Owner_1_", "PacketView", "1", "$owner->{name}") . '</i>';
            }
        }
        catch Error with
        {
            $ruleText = SF::i18n::get_pm_msg("Rule Unavailable", "PacketView");
        };
    }

    $retString  = "<div class=section id=section_$key>";
    $retString .= "<h2 onclick=toggleSection('section_$key')>" . SF::i18n::get_pm_msg("Event Information", "PacketView") . '</h2>';
    
    #Endace_Integration_Patch
    my @gUrls = mkRequest($event);
    if ( $endace_integration_enable )
    {
	#foreach my $link (@gUrls)	
	foreach my $link_string (@gUrls)	
	{
	    my @arlink = split('@', $link_string);
	    my $printable_string = $arlink[0];
	    my $link = $arlink[1];
	    #my @probeip = split('/', $link);
	    
	    $retString 	.= '<br><a class="iconlink" target="_blank" style="float: left;" '
			. "href=\"$link\">"
			. '<img src="/img/icons/medium/endace_logo.png" style="height:31px;"> '
			. "$printable_string" . '</a><br><br>'
			if ( $var->{outputType} eq "HTML" );
	
	    #for my $i ( 0 .. $#{ $visualizations{$probeip[2]} } ) 
	    #{
	    #    $retString .= '<br><a class="iconlink" target="_blank" style="float: left;" '
	    #		. "href=\"$visualizations{$probeip[2]}[$i]\">"
	    #		. '<img src="/img/icons/medium/endace_vision_logo.png" style="height:31px;"> '
	    #		. "Analyze in Endace Vision at $probeip[2]" . '</a><br><br>'
	    #		if ( $var->{outputType} eq "HTML" );
	    #}
	    
	    #$retString .= '<br><a class="iconlink" target="_blank" style="float: left;" '
		#	. "href=\"$link\">"
		#	. '<img src="/img/icons/medium/endace_logo.png" style="height:31px;"> '
		#	. "Download PCAP from Endace probe $probeip[2]" . '</a><br><br>'
		#	. '<br><a class="iconlink" target="_blank" style="float: left;" '
		#	. "href=\"$visualizations{$probeip[2]}[0]\">"
		#	. '<img src="/img/icons/medium/endace_vision_logo.png" style="height:31px;"> '
		#	. "Analyze in Endace Vision $probeip[2]" . '</a><br><br>'  
		#	if ( $var->{outputType} eq "HTML" );
	}
	
	#foreach my $vlink ( @vizlinks ) 
	foreach my $link_ref ( @vizlinks ) 
	{
	    my @alinkref = split("@",$link_ref) ; 
	    my  $rotname = $alinkref[0];
	    my  $vlink = $alinkref[1];
	    
	    my @probeip = split('/', $vlink);
	    
	    $retString .= '<br><a class="iconlink" target="_blank" style="float: left;" '
	    		. "href=\"$vlink\">"
	    		. '<img src="/img/icons/medium/endace_vision_logo.png" style="height:31px;"> '
	    		. "Analyze in Endace Vision for RotationFile <b> $rotname </b>" . '</a><br><br>'
	    		if ( $var->{outputType} eq "HTML" );
	}
	    
	
    }
    #Endace_Integration_Patch
    
    $retString .= "<table class='simple-table content'>";
    $retString .= "<tr><th align=left>" . SF::i18n::get_pm_msg("Event", "PacketView") . "</th><td>$event->{display_text}</td></tr>";
    $retString .= "<tr><th align=left>" . SF::i18n::get_pm_msg("Timestamp", "PacketView") . "</th><td>$timeString</td></tr>";

    # Dynamic Columns
    foreach my $column (grep { $_->{'display'} } @{$event->{'columns'}})
    {
        $retString .= "<tr><th align=left>$column->{'display_name'}</th><td>$column->{'display_data'}</td></tr>";
    }

    # Rule
    if ($ruleText)
    {
        $retString .= "<tr><th align=left>".SF::i18n::get_pm_msg("Rule", "PacketView")."</th><td>$ruleText</td></tr>";
    }

    # Summary
    if ($rule_obj && $event->{'summary'})
    {
        my $summaryText = join(' ', @{$event->{'summary'}});
        $summaryText =~ s/\s+/ /g;
        $retString .= "<tr><th align=left>".SF::i18n::get_pm_msg("Summary", "PacketView")."</th><td>$summaryText</td></tr>";
    }

    $retString .= '</table></div>';
    return $retString if ( $var->{'outputType'} eq "HTML_REPORT" );

    # Actions
    if ($rule_obj)
    {
        my $canEditPolicy = 0;
        my $policy;
        try
        {
            my ($policy_rev) = map {$_->{data}} grep {$_->{name} eq 'ips_policy'} @{$event->{columns}};
            my $policy_uuid = SF::SFDBI::Fetch('SELECT uuid FROM EOAttributes WHERE attribute = "name" AND revision_bin = UNHEX(?)', $policy_rev)->[0];
            $policy = SF::Intrusion::Policy->new($policy_uuid) if ( $policy_uuid );
            $var->{policy_name} = $policy->name;
            $event->{policy_name} = $policy->name;
            $event->{policy_rev} = $policy_rev;
        }
        catch Error with
        {
            $policy = undef;
        };

        $key = "$var->{eventCount}_0_1";
        if ($var->{outputType} eq "HTML") {
            $retString .= "<div class='collapsed section' id=section_$key>";
            $retString .= "<h2 onclick=toggleSection('section_$key')>" . SF::i18n::get_pm_msg("Actions", "PacketView") . "</h2>";
        }
        else {
            $retString .= "<div class='section'>";
            $retString .= "<h2>" . SF::i18n::get_pm_msg("Actions", "PacketView") . "</h2>";
        }
        $retString .= "<table class='simple-table content'>";
        $retString .= "<tr><th align=left>" . SF::i18n::get_pm_msg("Rule Actions", "PacketView") . "</th><td>";
        $retString .= getURLStrings($event) unless SF::Global::isStandardIPS();

        if (SF::Permission::hasPermission("intrusion_policy.modify") && $rule_obj->{data}{sid} != 135)
        {
            my $rule_pref = $event->{'deactivate_pref'};

            if (!$rule_pref || $rule_pref eq 'Ask' || $rule_pref eq 'Current Policy')
            {
                if (defined $policy && SF::EOHandler::checkAuthority($policy->authority))
                {
                    $retString .= "<br>";
                    my $current_rule_status = $policy->rule($$rule_obj{uuid});
                    if (defined $current_rule_status && $current_rule_status->{state})
                    {
                        $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('deactivate_current', $var, $event) . "'>";
                        $retString .= SF::i18n::get_format_pm_msg("rule_disable_current_policy_1", "PacketView", 
                                          1, SF::UIUtils::encodeHTML($var->{policy_name})
                                      );
                        $retString .= "</a>";
                    }
                    if (!defined $current_rule_status || $current_rule_status->{state} != 1)
                    {
                        $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('activate_current', $var, $event) . "'>";
                        $retString .= SF::i18n::get_format_pm_msg("rule_enable_current_policy_1", "PacketView", 
                                          1, SF::UIUtils::encodeHTML($var->{policy_name})
                                      );
                        $retString .= "</a>";
                    }
                    if ($policy->is_inline && (!defined $current_rule_status || $current_rule_status->{state} != 2))
                    {
                        $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('block_current', $var, $event) . "'>";
                        $retString .= SF::i18n::get_format_pm_msg("rule_drop_current_policy_1", "PacketView", 
                                          1, SF::UIUtils::encodeHTML($var->{policy_name})
                                      );
                        $retString .= "</a>";
                    }
                }
            }

            if (!$rule_pref || $rule_pref eq 'Ask' || $rule_pref eq 'All Policies')
            {
                $retString .= "<br>";

                $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('deactivate_all', $var, $event) . "'>";
                $retString .= SF::i18n::get_pm_msg("rule_disable_all_local_policy", "PacketView");
                $retString .= "</a>";

                $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('activate_all', $var, $event) . "'>";
                $retString .= SF::i18n::get_pm_msg("rule_enable_all_local_policy", "PacketView");
                $retString .= "</a>";

                $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('block_all', $var, $event) . "'>";
                $retString .= SF::i18n::get_pm_msg("rule_drop_all_local_policy", "PacketView");
                $retString .= "</a>";
            }
        }

        $retString .= '<br><br></td></tr>';

        if (SF::Permission::hasPermission("intrusion_policy.modify"))
        {
            $retString .= getHtmlText($canEditPolicy, $var, $event);
        }

        $retString .= '</table></div>';
    }

    return $retString;
}

sub getHtmlText
{
    my ($canEditPolicy, $var, $event) = @_;
    my $key = "$var->{eventCount}_0_1";
    my $threshold_option = SF::i18n::get_pm_msg("Set Thresholding Options", "PacketView");
    my $suppression_option = SF::i18n::get_pm_msg("Set Suppression Options", "PacketView");
    my $current_policy = SF::UIUtils::encodeHTML(SF::i18n::get_format_pm_msg("in_current_policy_1", "PacketView", ("1", "$var->{policy_name}")));
    my $local_policy = SF::i18n::get_pm_msg("local_policy", "PacketView");
    my $type_str = SF::i18n::get_pm_msg("Type", "PacketView");
    my $limit_str = SF::i18n::get_pm_msg("limit", "PacketView");
    my $threshold_str = SF::i18n::get_pm_msg("threshold", "PacketView");
    my $both_str = SF::i18n::get_pm_msg("both", "PacketView");
    my $rule_str = SF::i18n::get_pm_msg("rule", "PacketView");
    my $track_by_str = SF::i18n::get_pm_msg("track_by", "PacketView");
    my $source_ip = SF::i18n::get_pm_msg("source_ip", "PacketView");
    my $destination_ip = SF::i18n::get_pm_msg("destination_ip", "PacketView");
    my $count_str = SF::i18n::get_pm_msg("count_str", "PacketView");
    my $seconds_str = SF::i18n::get_pm_msg("seconds", "PacketView");
    my $overwrite_str = SF::i18n::get_pm_msg("overwrite", "PacketView");
    my $save_thresholding = SF::i18n::get_pm_msg("save_thresholding", "PacketView");
    my $save_suppression = SF::i18n::get_pm_msg("save_suppression", "PacketView");
    my $ip_cidr = SF::i18n::get_pm_msg("ip_cidr", "PacketView");
    my $retString=<<EOF;
       <tr><th align=left>$threshold_option</th><td>
<input type=hidden name="ts_gen_id" value="$event->{gen}">
<input type=hidden name="ts_sid" value="$event->{sid}">
<input type=hidden name="ts_rule_uuid" value="$event->{rule_uuid}">
<input type=hidden name="ts_de_int_id" value="$event->{sensor}">
<input type=hidden name="policy_rev" value="$event->{policy_rev}">
EOF
    if ( $canEditPolicy )
    {
        $retString .=<<EOF;
<a class="data" href="#" onClick="toggleThreshSuppBlocks('thresh_current');return false;"><img id="thresh_current_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$current_policy</b></br>
<div id='thresh_current' style="display: none;"><div class='vertical-form'>
  <table>
    <tr>
      <th align=left>$type_str</th>
      <td>
        <label><input type="radio" name="thresh_current_type" value="limit"/>$limit_str</label>
        <label><input type="radio" name="thresh_current_type" value="threshold"/>$threshold_str</label>
        <label><input type="radio" name="thresh_current_type" value="both"/>$both_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" name="thresh_current_track" value="by_src"/>$source_ip</label>
        <label><input type="radio" name="thresh_current_track" value="by_dst"/>$destination_ip</label>
      </td>
    </tr>
    <tr>
      <th align=left>$count_str</th>
      <td><input type="text" name="thresh_current_count"></td>
    </tr>
    <tr>
      <th align=left>$seconds_str</th>
      <td><input type="text" name="thresh_current_seconds"></td>
    </tr>
    <tr>
      <td></td>
      <td><label><input type="checkbox" name="thresh_current_override" value="1">$overwrite_str</label></td>
    </tr>
    <tr>
      <td></td>
      <td><input class=primary type="submit" name="action_thresh_current" value='$save_thresholding'/></td>
    </tr>
  </table>
</div></div><br>
EOF
    }

        $retString .=<<EOF;
<a class=data href="#" onClick="toggleThreshSuppBlocks('thresh_all'); return false;"><img id="thresh_all_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$local_policy</b></br>
<div id='thresh_all' style="display: none;">
<div class='vertical-form'>
  <table>
    <tr>
      <th align=left>$type_str</th>
      <td>
        <label><input type="radio" name="thresh_all_type" value="limit"/>$limit_str</label>
        <label><input type="radio" name="thresh_all_type" value="threshold"/>$threshold_str</label>
        <label><input type="radio" name="thresh_all_type" value="both"/>$both_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" name="thresh_all_track" value="by_src"/>$source_ip</label>
        <label><input type="radio" name="thresh_all_track" value="by_dst"/>$destination_ip</label>
      </td>
    </tr>
    <tr>
      <th align=left>$count_str</th>
      <td><input type="text" name="thresh_all_count"></td>
    </tr>
    <tr>
      <th align=left>$seconds_str</th>
      <td><input type="text" name="thresh_all_seconds"></td>
    </tr>
    <tr>
      <td></td>
      <td><label><input type="checkbox" name="thresh_all_override" value="1">$overwrite_str</label></td>
    </tr>
    <tr>
      <td></td>
      <td><input class=primary type="submit" name="action_thresh_all" value='$save_thresholding'/></td>
    </tr>
  </table>
</div></div>
</td></tr>
<tr><th align=left>$suppression_option</th><td>
EOF

    if ( $canEditPolicy )
    {
        $retString .=<<EOF;
              <a class=data href="#" onClick="toggleThreshSuppBlocks('suppress_current'); return false;"><img id="suppress_current_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$current_policy</b></br>
<div id="suppress_current" style="display: none;"><div class=vertical-form>
  <table>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_current1');" name="suppress_current_track" value="by_src">$source_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_current2');" name="suppress_current_track" value="by_dst">$destination_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_current3');" name="suppress_current_track" value="none" checked>$rule_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$ip_cidr</th>
      <td>
        <input id="suppress_current1" type="text" style="display:none"  value="">
        <input id="suppress_current2" type="text" style="display:none" value="">
        <input id="suppress_current3" type="text" name="suppress_current_address" disabled="yes">
      </td>
    </tr>
    <tr>
      <td></td>
      <td>
        <input class=primary type="submit" name="action_suppress_current" value='$save_suppression'/>
      </td>
    </tr>
  </table>
</div></div><br>
EOF
    }
        $retString .=<<EOF;
              <a class=data href="#" onClick="toggleThreshSuppBlocks('suppress_all'); return false;"><img id="suppress_all_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$local_policy</b></br>
<div id="suppress_all" style="display: none;"><div class=vertical-form>
  <table>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_all1');" name="suppress_all_track" value="by_src">$source_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_all2');" name="suppress_all_track" value="by_dst">$destination_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_all3');" name="suppress_all_track" value="none" checked>$rule_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$ip_cidr</th>
      <td>
        <input id="suppress_all1" type="text" style="display:none"  value="">
        <input id="suppress_all2" type="text" style="display:none" value="">
        <input id="suppress_all3" type="text" disabled="yes" name="suppress_all_address">
      </td>
    </tr>
    <tr>
      <td></td>
      <td><input class=primary type="submit" name="action_suppress_all" value='$save_suppression'/></td>
    </tr>
  </table>
</div></div></td></tr>
EOF

    return $retString;
}

sub getCreateFormJavascript
{
    my ($cmd, $var, $event) = @_;
    my $tmp_cgi = new SF($var->{cgiSF});
    $tmp_cgi->delete('deactivate_current');
    $tmp_cgi->delete('activate_current');
    $tmp_cgi->delete('block_current');
    $tmp_cgi->delete('deactivate_all');
    $tmp_cgi->delete('activate_all');
    $tmp_cgi->delete('block_all');
    $tmp_cgi->param(-name => $cmd, -value => $event->{'sensor'} .':'. $event->{'gen'} .':'. $event->{'sid'});
    $tmp_cgi->param(-name => 'policy_rev', -value => $event->{'policy_rev'});
    my $urlString = SF::Global::isStandardIPS()
                  ? SF::Events::EventsUI::CreateFormJavascript($tmp_cgi, undef, $tmp_cgi->url( -absolute => 1, -query => 1 ))
                  : SF::Events::EventsUI::CreateFormJavascript($tmp_cgi);
    return $urlString;
}

sub getURLStrings
{
   my ($event) = @_;

   my $strURL = "";
   my $extBase = "";
   $extBase = "/extensions/ui-components" if (SF::Extensions::isInstalled("ui-components"));

   if (SF::Permission::hasPermission("intrusion_policy.modify"))
   {
       $strURL .= "<a class=data href=\"javascript:void(0)\" onClick=\"window.open('/DetectionPolicy/rules/edit.cgi?"
           . "wizard=1&rule_uuid=$event->{rule_uuid}&rev_uuid=$event->{rule_rev}','newwindow',"
           . "'height=900, width=900, toolbar=no, menubar=no, scrollbars=yes, "
           . "resizeable=yes, location=no, directories=no, status=no')\">"
           . SF::i18n::get_pm_msg("Edit", "PacketView") . "</a><br>";
   }

   $strURL .= "<a class=data href=\"javascript:void(0)\" onClick=\"window.open('/events/context.cgi?"
       . "Type=Rules&behavior=documentation&uuid=$event->{rule_uuid}&rev_uuid=$event->{rule_rev}','newwindow',"
       . "'height=900, width=900, toolbar=no, menubar=no, scrollbars=yes, "
       . "resizeable=yes, location=no, directories=no, status=no')\">"
       . SF::i18n::get_pm_msg("View Documentation", "PacketView") . "</a>";

   if (SF::Permission::hasPermission("intrusion_policy.modify"))
   {
       $strURL .= "<br><a class=data href=\"javascript:void(0)\" onClick=\"window.open('/DetectionPolicy/rules/comment.cgi?"
           . "gen=$event->{gen}&sid=$event->{sid}&rev=$event->{rev}&from=packet&nomenu=1&notitle=1','newwindow',"
           . "'height=900, width=900, toolbar=no, menubar=no, scrollbars=yes, "
           . "resizeable=yes, location=no, directories=no, status=no')\">"
           . SF::i18n::get_pm_msg("Rule Comment", "PacketView") . "</a>";
   }

   return $strURL;
}


sub draw_packet
{
    my ( $packet, $var ) = @_;
    warn "draw_packet ** ". Dumper( $packet ) if ( $_debug && $_debug > 2 );

    my $frameData = draw_frame( $packet, $var );
    my $payloadData = draw_payload( $packet, {
            frame_id => $frameData->{frame_id},
            section_id => $frameData->{section_id},
            }, $var
        );

    my $frameID = $frameData->{frame_id};
    my $frameCount = $frameData->{frame_count};
    my $displayFrame = $frameData->{display_text};
    my $displayPayload = $payloadData->{display_text};
    my $ExpandAllStr = SF::i18n::get_pm_msg( "expand_all", "PacketView");
    my $expand = !$var || $var->{outputType} eq 'HTML' ? "<a class=iconlink id='$frameID' onclick='expandFrame( this, \"$frameID\" )'>$ExpandAllStr</a>" : "";
    my $html =<<EOF;

<div class=side-bar style='display:block; margin-bottom: 8px'>
  <h3 class=frame-header>FRAME $frameCount $expand</h3>
$displayFrame
$displayPayload
</div>
EOF
    if ( $expandState && $expandState == 2 )
    {
        $html .= "<script> expandSection('$frameID')</script>";
    }

    return $html;
}

sub draw_frame
{
    my ( $packet, $var ) = @_;
    warn " **draw_frame\n" if ( $_debug && $_debug > 2 );
    warn "Expand State $expandState\n" if ( $_debug );
    my @lines = @{$packet->{data}};
    my $retStruct;
    my $dc = $packet->{dc};
    my $frameCount = 1;
    $frameCount = $1 if ($lines[0] =~ /Frame (\d+)/);
    my $frameID = $packet->{eventCount} ."_" . $frameCount;
    my $section_id = 0;
    my $in_tbody;
    my $lineCount = @lines;
    my $html = "";
    for( my $pos = 0; $pos < $lineCount; $pos++ )
    {
        my $line = SF::UIUtils::encodeHTML($lines[$pos]);
        if ( $pos == 0)
        {
            $line =~ s/\(.*,\s*/\(/;
        }
        else
        {
            $line =~ s{ (\d+\.\d+\.\d+\.\d+)\s+\((\d+\.\d+\.\d+\.\d+)\) } { SF::Render::render("packetview.ip", $1, $dc) }gex;
            $line =~ s{ (\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\s+\((\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\) } { SF::Render::render("packetview.macaddr", $1) }gex;
        }

        if($line =~ /^(.*Time.*\s+)(\w+\s+\d+,\s+\d+\s+\d+:\d+:\d+)(\.\d+)?(.*)$/)
        {
            ## We add the TZ Offset because UnixDate assumes the time that is passed in is in the local TZ
            ## This will essentially get the correct UTC time. Local time will do the correct TZ conversions
            my $time = UnixDate($2, "%s");
            $line = $1 . strftime("%b %d, %Y %X", localtime($time) ) . $3;
        }

        if( $line =~ /^\s+/ )
        {
            my ($label, $value) = $line =~ /^\s*([\w\s]+)\s*:\s*(.*)$/;
            if( $pos+1 < $lineCount && $lines[$pos+1] =~ /^\s{5}/ )
            {
                # begins a sub section
                $html .= "<tr><th align=left>$label</th><td>";
                # walk subsections
                $html .= "<pre>";
                for( ; (($pos+1 < $lineCount) && ($lines[$pos+1] =~ /^\s{5}/)); $pos++ )
                {
                    my $newline = SF::UIUtils::encodeHTML($lines[$pos+1]);
                    $newline =~ s/^\s*//;
                    $newline =~ s{ (\d+\.\d+\.\d+\.\d+)\s+\((\d+\.\d+\.\d+\.\d+)\) } { SF::Render::render("packetview.ip", $1, $dc) }gex;
                    $newline =~ s{ (\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\s+\((\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\) } { SF::Render::render("packetview.macaddr", $1) }gex;
                    $html .= "$newline\n";
                }
                $html .= "</pre></td></tr>";
            }
            else
            {
                # normal section
                $html .= "<tr><th align=left>$label</th><td>$value</td></tr>";
            }
        }
        else
        {
            if ($line =~ /,/)
            {
                $line =~ s/,\s*/ <span class="light">(/;
                $line .= ')</span>';
            }
            else
            {
                $line =~ s/\((.*)\)\s*$/ <span class="light">($1)/;
            }
            $html .= "</table></div>" if $in_tbody;
            $section_id++;
            if ($var->{outputType} eq "HTML") {
                $html .= "<div class='collapsed section' id=section_${frameID}_${section_id}>";
                $html .= "<h3 onclick=toggleSection('section_${frameID}_${section_id}')>$line</h3>";
            }
            else {
                $html .= "<div class='section'>";
                $html .= "<h3>$line</h3>";
            }
            $html .= "<table class='simple-table content'>";
            $in_tbody = 1;
        }
    }
    $html .= "</table></div>" if $in_tbody;
    $section_id++;
    $retStruct->{frame_count} = ${frameCount};
    $retStruct->{frame_id} = ${frameID};
    $retStruct->{section_id} = ${section_id};
    $retStruct->{display_text} = $html;
    return $retStruct;
}

sub convertText
{
    my ( $hexText) = @_;
    #
    # Convert hex into unsigned char, and replace no-printable character with .
    #
    $hexText =~ s/\s+//g;
    $hexText =~ s/([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
    $hexText =~ s/[^\n\r[:print:]]/./g;
    return SF::UIUtils::encodeHTML( $hexText );
}

sub draw_payload
{
    my ( $packet, $options, $var ) = @_;
    warn " **draw_payload\n". Dumper( $options ) if ( $_debug && $_debug > 2 );

    my @lines = @{$packet->{payload}};
    my $html;
    my $frameID = $options->{frame_id};
    my $sectionID = $options->{section_id};

    my @toExpand;
    my @payloadStruct;
    my $asBold = 0;
    my $htmlPacket;
    my $htmlData;
    my $numSection;
    my $hexSection;
    my $ascSection;
    my $dataSection;
    my $completePacket;

    # No Section Titles
    if ( $lines[0] =~ /^\S{4,}\s+/ )
    {
        foreach my $line ( @lines )
        {
            my ( $num, $hex, $asc ) = split( /\s\s+/, $line, 3);
            $dataSection .= $hex;
            $numSection .= SF::UIUtils::encodeHTML($num);
            $hexSection .= SF::UIUtils::encodeHTML($hex);
            $ascSection .= SF::UIUtils::encodeHTML($asc);
            $completePacket .= '<tr><td><pre>'.SF::UIUtils::encodeHTML($num).'</pre></td>'.
                                   '<td style="color:gray;"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($hex).'</pre></td>'.
                                   '<td style="color:gray;"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($asc).'</pre></td>'.
                               '</tr>';
        }

        $dataSection = convertText( $dataSection );
        $htmlData = "<pre>$dataSection</pre>";
        $htmlPacket = "<table>\n\t$completePacket\n</table>";
    }

    # Section Titles
    else
    {
        my $sectionTitle;
        foreach my $line ( @lines )
        {
            next if ( !$line );
            # Packet Data
            if ( $line =~ /^\S{4,}\s+/ )
            {
                my ( $num, $hex, $asc ) = split( /\s\s+/, $line, 3);
                $dataSection .= $hex;
                $numSection .= SF::UIUtils::encodeHTML($num);
                $hexSection .= SF::UIUtils::encodeHTML($hex);
                $ascSection .= SF::UIUtils::encodeHTML($asc);
                $completePacket .= '<tr><td><pre>'.SF::UIUtils::encodeHTML($num).'</pre></td>'.
                                       '<td style="color:gray"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($hex).'</pre></td>'.
                                       '<td style="color:gray;"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($asc).'</pre></td>'.
                                   '</tr>';
            }
            # Section Title 
            else
            {
                if ( $sectionTitle )
                {
                    $dataSection = convertText( $dataSection );
                    $htmlData .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
                      <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
                      <div class=content><pre>$dataSection</pre></div>
                    </div>";

                    push @toExpand, $sectionID if ( $expandState && $expandState == 3 );
                    $sectionID++;
                    $htmlPacket .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
                      <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
                      <div class=content><table>$completePacket</table></div>
                    </div>";

                    push @toExpand, $sectionID if ( $expandState && $expandState == 1 );
                    $sectionID++;
                }
                $sectionTitle = $line;
                $numSection = "";
                $hexSection = "";
                $ascSection = "";
                $completePacket = "";
                $dataSection = "";
            }
        }

        if ( $sectionTitle )
        {
            $dataSection = convertText( $dataSection );
            $htmlData .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
              <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
              <div class=content><pre>$dataSection</pre></div>
            </div>";

            push @toExpand, $sectionID if ( $expandState && $expandState == 3 );
            $sectionID++;
            $htmlPacket .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
              <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
              <div class=content><table>$completePacket</table></div>
            </div>";

            push @toExpand, $sectionID if ( $expandState && $expandState == 1 );
            $sectionID++;
        }
    }

    #Force English text
    my $packetByte = SF::i18n::get_pm_msg("Packet Bytes", "PacketView", "en_US");
    my $packetData = SF::i18n::get_pm_msg("Packet Text", "PacketView", "en_US");

    if ($var->{outputType} eq "HTML") {
        $html =<<EOF;
<div class='collapsed section' id=section_${frameID}_${sectionID}>
  <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$packetData</h3>
  <div class=content>$htmlData</div>
</div>
EOF
    }
    else {
        $html =<<EOF;
<div class='section'>
  <h3>$packetData</h3>
  <div class=content>$htmlData</div>
</div>
EOF
    }

    push @toExpand, $sectionID if ( $expandState && $expandState == 3 );
    $sectionID++;

    if ($var->{outputType} eq "HTML") {
        $html .=<<EOF;
<div class='collapsed section' id=section_${frameID}_${sectionID}>
  <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$packetByte</h3>
  <div class=content>$htmlPacket</div>
</div>
EOF
    }
    else {
        $html .=<<EOF;
<div class='section'>
  <h3>$packetByte</h3>
  <div class=content>$htmlPacket</div>
</div>
EOF
    }

    push @toExpand, $sectionID if ( $expandState && $expandState == 1 );
    $sectionID++;
    if ( $expandState && scalar(@toExpand) )
    {
        foreach my $idx ( @toExpand )
        {
            $html .= "<script> expandSection('img_${frameID}_${idx}')</script>";
        }
    }
    $options->{section_id} = $sectionID;

    my $retStruct;
    $retStruct->{display_text} = $html;
    return $retStruct;
}

sub preRenderPortScan
{
    my ( $frame, $addDSTInfo, $eventInfo ) = @_;
    $frame->{payload} =~ s/\s//g;
    my $len = $frame->{DATALen} * 2;
    my $dc = $eventInfo->{'sensor'};
    $len = 0 - $len;
    my $info = substr( $frame->{payload}, $len );
    $info =~ s/([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
    if (( $info =~ /Open Port:/ ) && $addDSTInfo )
    {
        my $dstAddr = $frame->{DSTAddress};
        $info =~ s/Open Port:\s*/Open Port: $dstAddr /g;
    }
    $info =~ s{ (\d+\.\d+\.\d+\.\d+) } { SF::Render::render("packetview.ip", $1, $dc) }gex;

    $info =~ s{ (.*):\s } { SF::i18n::get_pm_msg("$1", "PacketView").": " }gex;
    $frame->{display_text} = $info;
}

sub processPortscan
{
    my ( $var, $eventInfo ) = @_;
    my $inPayload = 0;
    my $frameCount = 0;
    my $frameData;
    foreach my $line ( @{$var->{port_scan_data}} )
    {
        if ( $inPayload == 1)
        {
            if( $line =~ /^$/ )
            {
                preRenderPortScan( $frameData, $var->{addDSTInfo}, $eventInfo);
                $inPayload = 0;
            }
            else
            {
                $frameData->{payload} .= substr($line, 6, 48);
            }
        }
        elsif ( $line =~ /^$/ )
        {
            $inPayload = 1;
            $frameData->{payload} = "";
        }
        elsif ( $line =~ /^Frame/ )
        {
            $frameData = {};
            $var->{data}[$frameCount++] = $frameData;
            $inPayload = 0;
        }
        elsif ( $line =~ /^Internet Protocol/ )
        {

            $line =~ /\((.+)\).*\((.+)\)/;
            $frameData->{SRCAddress} = $1;
            $frameData->{DSTAddress} = $2;
        }
        elsif ( $line =~ /^Data/ )
        {
            $line =~ /Data .(\d+)/;
            $frameData->{DATALen} = $1;
        }
    }
    preRenderPortScan( $frameData, $var->{addDSTInfo}, $eventInfo ) if ( !$frameData->{display_text} );
    warn Dumper( $eventInfo ) if ( $_debug && $_debug > 2 );
    warn Dumper( $var->{data} ) if ( $_debug && $_debug > 2 );
    push @{$var->{display_list}}, "<tr><th class=title>"
        . SF::i18n::get_pm_msg("Portscan Information", "PacketView") . "</th></tr>";

    my  $deName = SF::i18n::get_pm_msg("Detection Engine", "PacketView") . ":"
        . SF::Render::render( 'detection_engine_id', $eventInfo->{'sensor'} );
    $deName .= " (Software Agent) " if ( SF::Sensor::isAgentDE( split /:/, $eventInfo->{'sensor'} ));

    my $displayText;
    push @{$displayText}, $deName;
    push @{$displayText}, SF::i18n::get_pm_msg("Time", "PacketView") . ":"
        . SF::Render::render( 'time', $eventInfo->{tvsec} );
    push @{$displayText}, SF::i18n::get_pm_msg("Message", "PacketView") . ":"
        . $eventInfo->{display_text};

    my $idx;
    for ($idx = 0; $idx < scalar( @{$var->{data}} ); $idx++ )
    {
        my $frameData = $var->{data}[$idx];
        if ( $idx == 0 )
        {
            push @{$displayText}, SF::i18n::get_pm_msg("Source IP", "PacketView"). ":"
                . SF::Render::render("packetview.ip", $frameData->{SRCAddress}, $eventInfo->{'sensor'});
            push @{$displayText}, SF::i18n::get_pm_msg("Destination IP", "PacketView"). ":"
                . SF::Render::render("packetview.ip", $frameData->{DSTAddress}, $eventInfo->{'sensor'});
        }
        push @{$displayText}, split( "\n", $frameData->{display_text} );
    }
    warn Dumper( $displayText ) if ( $_debug && $_debug > 2 );

    for ( $idx = 0; $idx < scalar( @{$displayText} ); $idx++ )
    {
        my ($head, $text) = split( /\s*:\s*/, $displayText->[$idx], 2 );
        push @{$var->{display_list}}, "<tr><td> <b>$head :</b> $text</td></tr>";
    }
    return 1;
}

1;
