#!/usr/bin/perl

package SF::PacketView;

# -- See .pre-flyloader for original file
use FlyLoader;
use strict;
use File::Temp qw(tempfile tempdir);
use Data::Dumper;
use Error qw(:try);
use Date::Manip;
use POSIX;
use URI::Escape;

our $_debug;
our $imsConf = SF::Util::load_ims_conf();
$_debug = $imsConf->{DEBUG_PACKET_VIEW};
our $expandState = SF::UserPreferences::get_preference_value("Events","Expand packet view by default") || 0;

my $csvoutput = 0;
my $hidelinks = 0;

my $imgpath = "/img";

sub setRelativePath
{
  my ($pref) = @_;

  if ($pref)
  {
    $imgpath = "./img";
  }
  else
  {
    $imgpath = "/img";
  }
}

# This function is used by the automation team for product testing.
# If you change the arguments, output, or functionality of this function
# please open a bug:
#
# Product: Automation
# Component: ATF:Framework
# Subject should start with: SF_API Changes
#
# Quick link below:
# https://bugzilla.sourcefire.com/enter_bug.cgi?product=Automation&component=ATF:Framework
sub getPacketInformationForEvent
{
    my ( $query, $eventInfo, $options ) = @_; #, $eventData
    warn " ** SF::PacketView::getPacketInformationForEvent\n" if ( $_debug );
    warn Dumper( $eventInfo ) if ( $_debug && $_debug > 1 );

    my $gensid = $eventInfo->{gidsid};
    my $adddstinfo = 0;
    if ( $eventInfo->{gidsid} && $eventInfo->{gen} && $eventInfo->{sid} )
    {
        $adddstinfo = 1 if (($eventInfo->{gen} == 122) &&
            (( $eventInfo->{sid} == 3 ) || ( $eventInfo->{sid} == 7 )));
    }
    my $sandBox = undef;
    if ( $_debug && $_debug > 2 )
    {
        $sandBox = tempdir('PacketView_XXXX',
            DIR => SF::Reloc::RelocateFilename('/var/tmp'));
        warn "relocated sandbox $sandBox\n";
    }
    else
    {
        $sandBox = tempdir('PacketView_XXXX',
            DIR => SF::Reloc::RelocateFilename('/var/tmp'), CLEANUP => 1);
    }
    $sandBox = SF::Reloc::UnRelocateFilename( $sandBox );
    warn "    Sandbox location $sandBox\n" if ( $_debug );

    my $var;
    $var->{cgiSF} = $query;
    $var->{sandBox} = $sandBox;
    $var->{pcapPath} = $sandBox . "/pcap";
    $var->{addDSTInfo} = $adddstinfo;
    $var->{outputType} = "HTML" unless($options);
    $var->{outputType} = "PDF_REPORT" if ($options && $options->{pdf});
    $var->{outputType} = "HTML_REPORT" if ($options && $options->{html});
    $var->{eventCount} = ($options && $options->{eventCount})? $options->{eventCount} : 1;
    $var->{username} = (defined $options && defined $options->{username} && $options->{username} ne '') ? $options->{username} : undef;
    my $pcap_generated = SF::PacketRender::EventPacketsToPcap({sensor_id => $eventInfo->{sensor},
                                                      event_id  => $eventInfo->{event_id},
                                                      event_sec => $eventInfo->{tvsec},
                                                      pcap_file => $var->{pcapPath}});
    warn "    PCAP export status : $pcap_generated\n" if ( $_debug );
    #
    # This is the filter we pass to tethereal for cmg's special packets
    #
#    my $packet_info_filter_expression = "(eth.src==4D:41:43:44:41:44 && eth.dst == 4D:41:43:44:41:44 && eth.type==0x0800 && ip.ttl==0)";
#    system("/usr/bin/tethereal -Vnxr $sandBox/pcap \"$packet_info_filter_expression\" > $sandBox/tethereal.out");

    my $retString;
    try
    {
        my $data = SF::System::tshark( tempDir => $sandBox, decode_type => "port_scan" ) if($pcap_generated);
        warn Dumper( $data ) if ( $_debug && $_debug > 2 );

        if ( $data->{stdout} )
        {
            if ( $var->{outputType} =~ /HTML/ )
            {
            my @lines = split( "\n", $data->{stdout} );
            $var->{port_scan_data} = \@lines;
            $var->{packet_data} = undef;
            processPortscan( $var, $eventInfo );
            $retString = join "\n", @{$var->{display_list}};
                $retString = "<table class=simple-table>"
                    . $retString . "</table>";
            }
            else
            {
                $retString = $data->{stdout};
            }
        }
        else
        {
            $data = SF::System::tshark( tempDir => $sandBox, decode_type => "packet" ) if($pcap_generated);
            while ( $data->{stdout} =~ /\\[nr]$/m )
            {
                $data->{stdout} =~ s/\\[nr]$//gm;
            }
            warn Dumper( $data ) if ( $_debug && $_debug > 2 );
            my @lines = split( "\n", $data->{stdout} );
            if ( $var->{outputType} =~ /HTML/ )
            {
                $var->{packet_data} = \@lines;
                $var->{port_scan_data} = undef;
                $var->{showAll} = 1;
                #$var->{eventData} = $eventData;
                processPacket( $var, $eventInfo );
            }
            else
            {
                $var->{display_string}  = draw_event( $var, $eventInfo );
                $var->{display_string} .= "Packet Information\n";
                $var->{display_string} .= $data->{stdout};
            }
            $retString = $var->{display_string};
        }
    }
    catch Error with
    {
        my $E = shift;
        warn Dumper( $E ) if ( $_debug );
        warn Dumper( $E );
        warn "Caught Exception ** while decoding packet - IGNORE\n";
        $retString = "";
    };

    warn "getPacketInformationForEvent - rendering text\n$retString\n" if ( $_debug );
    return $retString;
}

sub initalJavascript
{
    my ($var) = @_;

    return ""  if ( $var->{eventCount} != 1);

    my $expand_all = SF::i18n::get_pm_msg( "expand_all", "PacketView");
    my $hide_all = SF::i18n::get_pm_msg( "hide_all", "PacketView");

    my $scriptText =<<END;
<style>
    pre {
        margin    : 0;
        font-size : 8pt !important;
    }

    .section {
        margin-bottom : 8px;
    }

    .section h2 {
        display       : inline-block;
        padding-right : 12px;
        margin        : 4px -12px 4px 0px;
    }

    .section h2 b {
        font-weight : normal;
    }

    .section th {
        width        : 160px;
        padding-left : 0px !important;
    }

    .side-bar .section th {
        width : 200px;
    }

    .empty.section h2 {
        background : none;
        cursor     : auto;
    }
</style>
END

    if ($var->{outputType} eq 'HTML') {
        $scriptText.=<<END;
<style>
    .section h2 {
        cursor        : pointer;
        background    : url(/img/triangle-down-red.png) 0 4px no-repeat;
        background-position : right;
    }

    .side-bar .section .content {
        margin-left : 12px;
    }

    .side-bar .section h3 {
        display      : inline-block;
        padding-left : 12px;
        margin       : 0px 0px 0px 0px;
    }

    .side-bar .section h3 {
        cursor       : pointer;
        background   : url(/img/triangle-down.png) 0 4px no-repeat;
    }

    .collapsed.section .content, .empty.section .content {
        display : none;
    }

    .collapsed.section h2 {
        background-image : url(/img/triangle-right-red.png);
    }

    .collapsed.section h3 {
        background-image : url(/img/triangle-right.png);
    }

    .frame-header {
        background : #dce6f4;
        margin     : -5px -5px 8px;
        padding    : 4px 17px;
    }
</style>

<script langauge=Javascript>
var maxSections = 0;

function toggleSection(id) {
    var section = document.getElementById(id);
    section.className = section.className === "collapsed section"
                      ? "section"
                      : "collapsed section";
}

function toggle( img )
{
    var section = document.getElementById(img.id.replace(/img/, "section"));

    if( img.src.match( "triangle-down.png" ) )
    {
        img.src = "$imgpath/triangle-right.png";
        section.style.display = "none";
    }
    else
    {
        img.src = "$imgpath/triangle-down.png";
        section.style.display = "";
    }
}

function initColumns()
{
    var divs = document.getElementsByTagName("div");
    for (var i = 0; i < divs.length; i++)
    {
        if (divs[i].className.indexOf("collapsible") > -1)
        {
            var img   = document.getElementById(divs[i].id.replace(/section/, "img"));
            var label = document.getElementById(divs[i].id.replace(/section/, "label"));
            if (img && label && divs[i].offsetHeight > label.offsetHeight)
            {
                divs[i].style.height = label.offsetHeight;
                img.style.display = "";
            }
        }
    }
}

function toggleColumn( img )
{
    var label   = document.getElementById(img.id.replace(/img/, "label"));
    var section = document.getElementById(img.id.replace(/img/, "section"));

    if( img.src.match( "triangle-down.png" ) )
    {
        img.src = "$imgpath/triangle-right.png";
        section.style.height = label.offsetHeight;
    }
    else
    {
        img.src = "$imgpath/triangle-down.png";
        section.style.height = "auto";
    }
}

function expandFrame( a, id )
{
    var tState;
    if( a.innerHTML == "$expand_all" )
    {
        tState = "section";
        a.innerHTML = "$hide_all";
    }
    else
    {
        tState = "collapsed section";
        a.innerHTML = "$expand_all";
    }
    var idx = 1;
    var tObj;
    while (tObj = document.getElementById( "section_" + id + "_" + idx )) {
        tObj.className = tState;
        idx++;
    }
}

function changePacketDataColour( id, type )
{
    if (type)
    {
        document.getElementById("HEX_" + id).style.backgroundColor = "yellow";
        document.getElementById("ASC_" + id).style.backgroundColor = "yellow";
    }
    else
    {
        document.getElementById("HEX_" + id).style.backgroundColor = null;
        document.getElementById("ASC_" + id).style.backgroundColor = null;
    }
}

var sectionID = new Array();
var userOption = $expandState;
function expandSection( str )
{
    sectionID[sectionID.length] = str;
}

function expandOnLoad()
{
    for (var i = 0; i < sectionID.length; i++)
    {
        var obj = document.getElementById( sectionID[i] );
        if ( obj && userOption && userOption != 2 ) { toggle( obj ); }
        if ( obj && userOption == 2 ) { expandFrame( obj, sectionID[i] ); }
    }
}
  // This routine handles hiding/un-hiding the threshold &
  // suppression stuff.
  //
  // If we open one, we close all of the others.
  //
  function toggleThreshSuppBlocks(blockId)
  {
    var block = document.getElementById(blockId);
    var blockStatus = block.style.display;

    if(blockId == "thresh_current" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_all");
       hideThreshSuppBlock("suppress_current");
       hideThreshSuppBlock("suppress_all");
       showThreshSuppBlock("thresh_current");
    }
    else if(blockId == "thresh_current" && blockStatus == "block")
    {
       hideThreshSuppBlock("thresh_current");
    }
    else if(blockId == "thresh_all" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_current");
       hideThreshSuppBlock("suppress_current");
       hideThreshSuppBlock("suppress_all");
       showThreshSuppBlock("thresh_all");
    }
    else if(blockId == "thresh_all" && blockStatus == "block")
    {
       hideThreshSuppBlock("thresh_all");
    }
    else if(blockId == "suppress_current" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_current");
       hideThreshSuppBlock("thresh_all");
       hideThreshSuppBlock("suppress_all");
       showThreshSuppBlock("suppress_current");
    }
    else if(blockId == "suppress_current" && blockStatus == "block")
    {
       hideThreshSuppBlock("suppress_current");
    }
    else if(blockId == "suppress_all" && blockStatus == "none")
    {
       hideThreshSuppBlock("thresh_current");
       hideThreshSuppBlock("thresh_all");
       hideThreshSuppBlock("suppress_current");
       showThreshSuppBlock("suppress_all");
    }
    else if(blockId == "suppress_all" && blockStatus == "block")
    {
       hideThreshSuppBlock("suppress_all");
    }
    else if(blockId == "suppress_current1" )
    {
    hideThreshSuppBlock("suppress_current2");
    hideThreshSuppBlock("suppress_current3");
    showThreshSuppBlock("suppress_current1");
    nameswap1("suppress_current1","suppress_current2","suppress_current3");
    }
    else if(blockId == "suppress_current2" )
    {
    hideThreshSuppBlock("suppress_current1");
    hideThreshSuppBlock("suppress_current3");
    showThreshSuppBlock("suppress_current2");
    nameswap1("suppress_current2","suppress_current1","suppress_current3");
    }
    else if(blockId == "suppress_current3" )
    {

    hideThreshSuppBlock("suppress_current1");
    hideThreshSuppBlock("suppress_current2");
    showThreshSuppBlock("suppress_current3");
    nameswap1("suppress_current3","suppress_current2","suppress_current1");
    }
    else if(blockId == "suppress_all1" && blockStatus != "block")
    {
    hideThreshSuppBlock("suppress_all2");
    hideThreshSuppBlock("suppress_all3");
    showThreshSuppBlock("suppress_all1");
    nameswap2("suppress_all1","suppress_all2","suppress_all3");
    }
    else if(blockId == "suppress_all2" && blockStatus != "block")
    {
    hideThreshSuppBlock("suppress_all1");
    hideThreshSuppBlock("suppress_all3");
    showThreshSuppBlock("suppress_all2");
    nameswap2("suppress_all2","suppress_all3","suppress_all1");
    }
    else if(blockId == "suppress_all3" && blockStatus != "block")
    {
    hideThreshSuppBlock("suppress_all2");
    hideThreshSuppBlock("suppress_all1");
    showThreshSuppBlock("suppress_all3");
    nameswap2("suppress_all3","suppress_all2","suppress_all1");
    }

   }
  function nameswap2(blockId1, blockId2, blockId3)
  {
     block1=document.getElementById(blockId1);
     block2=document.getElementById(blockId2);
     block3=document.getElementById(blockId3);
     block1.name="suppress_all_address"
     block2.name="suppress_not_all1"
     block3.name="suppress_not_all2"
  }
  function nameswap1(blockId1, blockId2, blockId3)
  {
     block1=document.getElementById(blockId1);
     block2=document.getElementById(blockId2);
     block3=document.getElementById(blockId3);
     block1.name="suppress_current_address"
     block2.name="suppress_not_current1"
     block3.name="suppress_not_current2"
  }
  //This shows a T & S block
  function showThreshSuppBlock(blockId)
  {

    // errors in here are OK.  catch and ignore them
    try
    {
       block = document.getElementById(blockId);
       blockImg = document.getElementById(blockId + "_img");

       block.style.display = "block";
       blockImg.src = "$imgpath/triangle-down.png";
    }
    catch (e)
    {
    //dbg_msg(e);
    }
  }


  // this hides a T & S block
  function hideThreshSuppBlock(blockId)
  {
    // Errors in here are OK. catch and ignore them
    try
    {
      block = document.getElementById(blockId);
      blockImg = document.getElementById(blockId + "_img");

      block.style.display = "none";
      blockImg.src = "$imgpath/triangle-right.png";
    }
    catch (e)
    {
      //dbg_msg(e);
    }
  }
END

        $scriptText .= "addOnLoad( initColumns ); addOnLoad( expandOnLoad );" if ( $var->{outputType} eq "HTML" );
        $scriptText .= "</script>";
    }

    return $scriptText;
}

sub processPacket
{
    my ( $var, $eventInfo ) = @_;

    my $timeString = SF::Render::render( "event_time", $eventInfo->{tvsec} );
    my $eventHtml  = draw_event( $var, $eventInfo );
    my $scriptText = initalJavascript($var);

    my $html = $scriptText.$eventHtml;

    if ( !scalar(@{$var->{packet_data}} ))
    {
        $html .= '<h2>'. SF::i18n::get_pm_msg("No Packet Information", "PacketView") .'</h2>';
        $var->{display_string} = $html;
        return 1;
    }
    $html .= '<h2>' . SF::i18n::get_pm_msg( "Packet Information", "PacketView") . '</h2>' if ( $var->{outputType} eq "HTML" );
    my $offset = 0;
    my $payload = [];
    my $data = [];
    my $numLines = scalar( @{$var->{packet_data}} );
    my $payloadMode = 0;
#
#FRAME 1 (610 bytes on wire, 610 bytes captured)
#    Arrival Time: May 29, 2008 21:27:12.825089000
#...
#0000  00 20 a6 4c 47 15 08 00 20 f1 2f 35 08 00 45 00   . .LG... ./5..E.
#
#                 --OR--
#
#FRAME 1 (610 bytes on wire, 610 bytes captured)
#    Arrival Time: May 29, 2008 21:27:12.825089000
#...
#
#Frame (610 bytes):
#
#0000  00 20 a6 4c 47 15 08 00 20 f1 2f 35 08 00 45 00   . .LG... ./5..E.
#...
#
#De-chunked entity body (290 bytes):
#
#0000  3c 21 44 4f 43 54 59 50 45 20 48 54 4d 4c 20 50   <!DOCTYPE HTML P
#...
#
#
#Definition of "Packet Byte "
# 1. No Title line and Lines Starting with number
# 2. Title line, any-number of blank lines and Lines Stating with number
# 3. Any number of type 2 format with blank line(s) in between.
#
#Note: Once Packet Byte starts, it goes till the end or start of next FRAME.
#
    for( my $pos = 0; $pos < $numLines; $pos++ )
    {
        my $line = $var->{packet_data}[$pos];
        next if( $line =~ /^\s*$/ );
        if ( $line =~ /^Frame.*bytes on wire.*bytes captured/ )
        {
            $html .= draw_packet({ showAll => $var->{showAll},
                data => $data,
                payload => $payload,
                eventCount => $var->{eventCount},
                dc => $eventInfo->{sensor},
                }, $var) if (scalar(@{$payload}) && scalar(@{$data}));
            $payload = [];
            $data = [ $line ];
            $offset = 0;
            $payloadMode = 0;
        }
        elsif ($payloadMode)
        {
            push @{$payload}, $line;
        }
        else
        {
            next if ( !$var->{showAll} );
            $line ="$1$2$3" if ( $line =~ /^(\s*)\[(.*)\](\s*)$/ );
            if ( $line =~/^\w/ )
            {
                my $loc = $pos+1;
                while( length($var->{packet_data}[$loc]) == 0 )
                {
                    $loc++;
                }
                if ( $var->{packet_data}[$loc] =~ /^\d+/ )
                {
                    $payloadMode = 1;
                    push @{$payload}, $line;
                    next;
                }
            }
            push @{$data}, $line;
        }
    }
    $html .= draw_packet({ showAll => $var->{showAll},
        data => $data,
        payload => $payload,
        eventCount => $var->{eventCount},
        dc => $eventInfo->{sensor},
        }, $var);
    $var->{display_string} = $html;
    return 1;
}

sub draw_event
{
    my ( $var, $event ) = @_;

    my $key        = "$var->{eventCount}_0_0";
    my $subkey     = 0;
    my $timeString = SF::Render::render( "event_time", $event->{tvsec} );

    my $columns = { map { $_->{'name'} => $_ } @{$event->{'columns'}} };

    # render all columns
    foreach my $column (@{$event->{'columns'}})
    {
        # use packetview IP renderer
        if ($column->{'type'} eq 'ip')
        {
            $column->{'type'} = 'packetview.ip';
        }

        # do not truncate column data
        elsif ($column->{'type'} eq 'long_string')
        {
            $column->{'type'} = 'string';
        }

        if (defined $column->{'data'} && $column->{'data'} =~ /:/)
        {
            my @render_args = ($column->{'type'}, $column->{'data'});
            $column->{'display_data'} = SF::Render::renderExtraText(@render_args) . SF::Render::render(@render_args);
        }
        else
        {
            $column->{'display_data'} = SF::Render::render($column->{'type'}, $column->{'data'});
        }

        if (!defined $column->{'display_data'} || $column->{'display_data'} =~ /^\s*$/)
        {
            $column->{'display'} = 0 unless $column->{'display'};
            $column->{'display_data'} = '<span style="color: gray;">' . SF::i18n::get_pm_msg('no_data', 'PacketView') . '</span>';
        }
        else
        {
            $column->{'display'} = 1;
            if (defined $column->{'dependencies'})
            {
                foreach my $dependency (grep { exists $columns->{$_} } @{$column->{'dependencies'}})
                {
                    $columns->{$dependency}{'display'} = 1;
                }
            }
        }
    }

    my $retString;

    if ($var->{'outputType'} eq "PDF_REPORT")
    {
        $retString = "Event Information\nEvent: $event->{display_text}, Timestamp: $timeString\n";
        foreach my $column (grep { $_->{'display'} } @{$event->{'columns'}})
        {
            $retString .= $column->{'display_name'} . ' : ' . $column->{'display_data'} . "\n";
        }
        return $retString;
    }

    my ($rule_obj, $ruleText);
    try
    {
        my $idsEventMsgMap    = SF::IDS::Events::Map::getMsgEntryBySID($event->{'sid'}, $event->{'gen'});
        $event->{'rule_uuid'} = $idsEventMsgMap->{'uuid'};
        $event->{'rule_rev'}  = $idsEventMsgMap->{'rev_uuid'};
        $rule_obj             = SF::EOHandler::loadObject($event->{'rule_uuid'});
    }
    catch Error with
    {
        $rule_obj = undef;
    };
    if ($event->{'gen'} != 138) # bug 78069 - hide rule text for GID 138 events
    {
        try
        {
            my $rule_rev    = SF::EOHandler::loadObject($event->{'rule_uuid'}, $event->{'rule_rev'});
            $event->{'sid'} = $rule_rev->{'data'}{'sid'};
            $event->{'gen'} = $rule_rev->{'data'}{'gid'};
            my $username = (defined $var and exists $var->{username} and defined $var->{username} and $var->{username} ne '') ? $var->{username} : undef;
            if ($event->{'sid'} >= 1000000 and !SF::Permission::hasAnyPermission(["local_rules","rules"], $username)) {
                $ruleText = "Rule text view disabled for sid: $event->{'sid'}";
#                $ruleText = "Rule text view disabled for sid: $event->{'sid'}<pre>var is ".Dumper($var)."\nevent is ".Dumper($event)."\nstacktrace is ".SF::Util::Stacktrace::ToString()."</pre>";
            } else {
                $ruleText = HTML::Entities::encode(SF::Intrusion::Rule::Parse::gen_rule($rule_rev->{'data'}));
            }
            unless ($rule_obj)
            {
                my $owner = SF::PeerManager::Peers::getPeerInfo($rule_rev->{'authority'});
                $ruleText .= '&nbsp;<i>' . SF::i18n::get_format_pm_msg("Remote_Owner_1_", "PacketView", "1", "$owner->{name}") . '</i>';
            }
        }
        catch Error with
        {
            $ruleText = SF::i18n::get_pm_msg("Rule Unavailable", "PacketView");
        };
    }

    $retString  = "<div class=section id=section_$key>";
    $retString .= "<h2 onclick=toggleSection('section_$key')>" . SF::i18n::get_pm_msg("Event Information", "PacketView") . '</h2>';
    $retString .= "<table class='simple-table content'>";
    $retString .= "<tr><th align=left>" . SF::i18n::get_pm_msg("Event", "PacketView") . "</th><td>$event->{display_text}</td></tr>";
    $retString .= "<tr><th align=left>" . SF::i18n::get_pm_msg("Timestamp", "PacketView") . "</th><td>$timeString</td></tr>";

    # Dynamic Columns
    foreach my $column (grep { $_->{'display'} } @{$event->{'columns'}})
    {
        $retString .= "<tr><th align=left>$column->{'display_name'}</th><td>$column->{'display_data'}</td></tr>";
    }

    # Rule
    if ($ruleText)
    {
        $retString .= "<tr><th align=left>".SF::i18n::get_pm_msg("Rule", "PacketView")."</th><td>$ruleText</td></tr>";
    }

    # Summary
    if ($rule_obj && $event->{'summary'})
    {
        my $summaryText = join(' ', @{$event->{'summary'}});
        $summaryText =~ s/\s+/ /g;
        $retString .= "<tr><th align=left>".SF::i18n::get_pm_msg("Summary", "PacketView")."</th><td>$summaryText</td></tr>";
    }

    $retString .= '</table></div>';
    return $retString if ( $var->{'outputType'} eq "HTML_REPORT" );

    # Actions
    if ($rule_obj)
    {
        my $canEditPolicy = 0;
        my $policy;
        try
        {
            my ($policy_rev) = map {$_->{data}} grep {$_->{name} eq 'ips_policy'} @{$event->{columns}};
            my $policy_uuid = SF::SFDBI::Fetch('SELECT uuid FROM EOAttributes WHERE attribute = "name" AND revision_bin = UNHEX(?)', $policy_rev)->[0];
            $policy = SF::Intrusion::Policy->new($policy_uuid) if ( $policy_uuid );
            $var->{policy_name} = $policy->name;
            $event->{policy_name} = $policy->name;
            $event->{policy_rev} = $policy_rev;
        }
        catch Error with
        {
            $policy = undef;
        };

        $key = "$var->{eventCount}_0_1";
        if ($var->{outputType} eq "HTML") {
            $retString .= "<div class='collapsed section' id=section_$key>";
            $retString .= "<h2 onclick=toggleSection('section_$key')>" . SF::i18n::get_pm_msg("Actions", "PacketView") . "</h2>";
        }
        else {
            $retString .= "<div class='section'>";
            $retString .= "<h2>" . SF::i18n::get_pm_msg("Actions", "PacketView") . "</h2>";
        }
        $retString .= "<table class='simple-table content'>";
        $retString .= "<tr><th align=left>" . SF::i18n::get_pm_msg("Rule Actions", "PacketView") . "</th><td>";
        $retString .= getURLStrings($event) unless SF::Global::isStandardIPS();

        if (SF::Permission::hasPermission("intrusion_policy.modify") && $rule_obj->{data}{sid} != 135)
        {
            my $rule_pref = $event->{'deactivate_pref'};

            if (!$rule_pref || $rule_pref eq 'Ask' || $rule_pref eq 'Current Policy')
            {
                if (defined $policy && SF::EOHandler::checkAuthority($policy->authority))
                {
                    $retString .= "<br>";
                    my $current_rule_status = $policy->rule($$rule_obj{uuid});
                    if (defined $current_rule_status && $current_rule_status->{state})
                    {
                        $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('deactivate_current', $var, $event) . "'>";
                        $retString .= SF::i18n::get_format_pm_msg("rule_disable_current_policy_1", "PacketView", 
                                          1, SF::UIUtils::encodeHTML($var->{policy_name})
                                      );
                        $retString .= "</a>";
                    }
                    if (!defined $current_rule_status || $current_rule_status->{state} != 1)
                    {
                        $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('activate_current', $var, $event) . "'>";
                        $retString .= SF::i18n::get_format_pm_msg("rule_enable_current_policy_1", "PacketView", 
                                          1, SF::UIUtils::encodeHTML($var->{policy_name})
                                      );
                        $retString .= "</a>";
                    }
                    if ($policy->is_inline && (!defined $current_rule_status || $current_rule_status->{state} != 2))
                    {
                        $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('block_current', $var, $event) . "'>";
                        $retString .= SF::i18n::get_format_pm_msg("rule_drop_current_policy_1", "PacketView", 
                                          1, SF::UIUtils::encodeHTML($var->{policy_name})
                                      );
                        $retString .= "</a>";
                    }
                }
            }

            if (!$rule_pref || $rule_pref eq 'Ask' || $rule_pref eq 'All Policies')
            {
                $retString .= "<br>";

                $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('deactivate_all', $var, $event) . "'>";
                $retString .= SF::i18n::get_pm_msg("rule_disable_all_local_policy", "PacketView");
                $retString .= "</a>";

                $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('activate_all', $var, $event) . "'>";
                $retString .= SF::i18n::get_pm_msg("rule_enable_all_local_policy", "PacketView");
                $retString .= "</a>";

                $retString .= "<br><a href='javascript:void(0)' class='data' onclick='" . getCreateFormJavascript('block_all', $var, $event) . "'>";
                $retString .= SF::i18n::get_pm_msg("rule_drop_all_local_policy", "PacketView");
                $retString .= "</a>";
            }
        }

        $retString .= '<br><br></td></tr>';

        if (SF::Permission::hasPermission("intrusion_policy.modify"))
        {
            $retString .= getHtmlText($canEditPolicy, $var, $event);
        }

        $retString .= '</table></div>';
    }

    return $retString;
}

sub getHtmlText
{
    my ($canEditPolicy, $var, $event) = @_;
    my $key = "$var->{eventCount}_0_1";
    my $threshold_option = SF::i18n::get_pm_msg("Set Thresholding Options", "PacketView");
    my $suppression_option = SF::i18n::get_pm_msg("Set Suppression Options", "PacketView");
    my $current_policy = SF::UIUtils::encodeHTML(SF::i18n::get_format_pm_msg("in_current_policy_1", "PacketView", ("1", "$var->{policy_name}")));
    my $local_policy = SF::i18n::get_pm_msg("local_policy", "PacketView");
    my $type_str = SF::i18n::get_pm_msg("Type", "PacketView");
    my $limit_str = SF::i18n::get_pm_msg("limit", "PacketView");
    my $threshold_str = SF::i18n::get_pm_msg("threshold", "PacketView");
    my $both_str = SF::i18n::get_pm_msg("both", "PacketView");
    my $rule_str = SF::i18n::get_pm_msg("rule", "PacketView");
    my $track_by_str = SF::i18n::get_pm_msg("track_by", "PacketView");
    my $source_ip = SF::i18n::get_pm_msg("source_ip", "PacketView");
    my $destination_ip = SF::i18n::get_pm_msg("destination_ip", "PacketView");
    my $count_str = SF::i18n::get_pm_msg("count_str", "PacketView");
    my $seconds_str = SF::i18n::get_pm_msg("seconds", "PacketView");
    my $overwrite_str = SF::i18n::get_pm_msg("overwrite", "PacketView");
    my $save_thresholding = SF::i18n::get_pm_msg("save_thresholding", "PacketView");
    my $save_suppression = SF::i18n::get_pm_msg("save_suppression", "PacketView");
    my $ip_cidr = SF::i18n::get_pm_msg("ip_cidr", "PacketView");
    my $retString=<<EOF;
       <tr><th align=left>$threshold_option</th><td>
<input type=hidden name="ts_gen_id" value="$event->{gen}">
<input type=hidden name="ts_sid" value="$event->{sid}">
<input type=hidden name="ts_rule_uuid" value="$event->{rule_uuid}">
<input type=hidden name="ts_de_int_id" value="$event->{sensor}">
<input type=hidden name="policy_rev" value="$event->{policy_rev}">
EOF
    if ( $canEditPolicy )
    {
        $retString .=<<EOF;
<a class="data" href="#" onClick="toggleThreshSuppBlocks('thresh_current');return false;"><img id="thresh_current_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$current_policy</b></br>
<div id='thresh_current' style="display: none;"><div class='vertical-form'>
  <table>
    <tr>
      <th align=left>$type_str</th>
      <td>
        <label><input type="radio" name="thresh_current_type" value="limit"/>$limit_str</label>
        <label><input type="radio" name="thresh_current_type" value="threshold"/>$threshold_str</label>
        <label><input type="radio" name="thresh_current_type" value="both"/>$both_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" name="thresh_current_track" value="by_src"/>$source_ip</label>
        <label><input type="radio" name="thresh_current_track" value="by_dst"/>$destination_ip</label>
      </td>
    </tr>
    <tr>
      <th align=left>$count_str</th>
      <td><input type="text" name="thresh_current_count"></td>
    </tr>
    <tr>
      <th align=left>$seconds_str</th>
      <td><input type="text" name="thresh_current_seconds"></td>
    </tr>
    <tr>
      <td></td>
      <td><label><input type="checkbox" name="thresh_current_override" value="1">$overwrite_str</label></td>
    </tr>
    <tr>
      <td></td>
      <td><input class=primary type="submit" name="action_thresh_current" value='$save_thresholding'/></td>
    </tr>
  </table>
</div></div><br>
EOF
    }

        $retString .=<<EOF;
<a class=data href="#" onClick="toggleThreshSuppBlocks('thresh_all'); return false;"><img id="thresh_all_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$local_policy</b></br>
<div id='thresh_all' style="display: none;">
<div class='vertical-form'>
  <table>
    <tr>
      <th align=left>$type_str</th>
      <td>
        <label><input type="radio" name="thresh_all_type" value="limit"/>$limit_str</label>
        <label><input type="radio" name="thresh_all_type" value="threshold"/>$threshold_str</label>
        <label><input type="radio" name="thresh_all_type" value="both"/>$both_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" name="thresh_all_track" value="by_src"/>$source_ip</label>
        <label><input type="radio" name="thresh_all_track" value="by_dst"/>$destination_ip</label>
      </td>
    </tr>
    <tr>
      <th align=left>$count_str</th>
      <td><input type="text" name="thresh_all_count"></td>
    </tr>
    <tr>
      <th align=left>$seconds_str</th>
      <td><input type="text" name="thresh_all_seconds"></td>
    </tr>
    <tr>
      <td></td>
      <td><label><input type="checkbox" name="thresh_all_override" value="1">$overwrite_str</label></td>
    </tr>
    <tr>
      <td></td>
      <td><input class=primary type="submit" name="action_thresh_all" value='$save_thresholding'/></td>
    </tr>
  </table>
</div></div>
</td></tr>
<tr><th align=left>$suppression_option</th><td>
EOF

    if ( $canEditPolicy )
    {
        $retString .=<<EOF;
              <a class=data href="#" onClick="toggleThreshSuppBlocks('suppress_current'); return false;"><img id="suppress_current_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$current_policy</b></br>
<div id="suppress_current" style="display: none;"><div class=vertical-form>
  <table>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_current1');" name="suppress_current_track" value="by_src">$source_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_current2');" name="suppress_current_track" value="by_dst">$destination_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_current3');" name="suppress_current_track" value="none" checked>$rule_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$ip_cidr</th>
      <td>
        <input id="suppress_current1" type="text" style="display:none"  value="">
        <input id="suppress_current2" type="text" style="display:none" value="">
        <input id="suppress_current3" type="text" name="suppress_current_address" disabled="yes">
      </td>
    </tr>
    <tr>
      <td></td>
      <td>
        <input class=primary type="submit" name="action_suppress_current" value='$save_suppression'/>
      </td>
    </tr>
  </table>
</div></div><br>
EOF
    }
        $retString .=<<EOF;
              <a class=data href="#" onClick="toggleThreshSuppBlocks('suppress_all'); return false;"><img id="suppress_all_img" src="$imgpath/triangle-right.png" style="cursor: pointer;"></a>&nbsp;<b>$local_policy</b></br>
<div id="suppress_all" style="display: none;"><div class=vertical-form>
  <table>
    <tr>
      <th align=left>$track_by_str</th>
      <td>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_all1');" name="suppress_all_track" value="by_src">$source_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_all2');" name="suppress_all_track" value="by_dst">$destination_ip</label>
        <label><input type="radio" onClick="toggleThreshSuppBlocks('suppress_all3');" name="suppress_all_track" value="none" checked>$rule_str</label>
      </td>
    </tr>
    <tr>
      <th align=left>$ip_cidr</th>
      <td>
        <input id="suppress_all1" type="text" style="display:none"  value="">
        <input id="suppress_all2" type="text" style="display:none" value="">
        <input id="suppress_all3" type="text" disabled="yes" name="suppress_all_address">
      </td>
    </tr>
    <tr>
      <td></td>
      <td><input class=primary type="submit" name="action_suppress_all" value='$save_suppression'/></td>
    </tr>
  </table>
</div></div></td></tr>
EOF

    return $retString;
}

sub getCreateFormJavascript
{
    my ($cmd, $var, $event) = @_;
    my $tmp_cgi = new SF($var->{cgiSF});
    $tmp_cgi->delete('deactivate_current');
    $tmp_cgi->delete('activate_current');
    $tmp_cgi->delete('block_current');
    $tmp_cgi->delete('deactivate_all');
    $tmp_cgi->delete('activate_all');
    $tmp_cgi->delete('block_all');
    $tmp_cgi->param(-name => $cmd, -value => $event->{'sensor'} .':'. $event->{'gen'} .':'. $event->{'sid'});
    $tmp_cgi->param(-name => 'policy_rev', -value => $event->{'policy_rev'});
    my $urlString = SF::Global::isStandardIPS()
                  ? SF::Events::EventsUI::CreateFormJavascript($tmp_cgi, undef, $tmp_cgi->url( -absolute => 1, -query => 1 ))
                  : SF::Events::EventsUI::CreateFormJavascript($tmp_cgi);
    return $urlString;
}

sub getURLStrings
{
   my ($event) = @_;

   my $strURL = "";
   my $extBase = "";
   $extBase = "/extensions/ui-components" if (SF::Extensions::isInstalled("ui-components"));

   if (SF::Permission::hasPermission("intrusion_policy.modify"))
   {
       $strURL .= "<a class=data href=\"javascript:void(0)\" onClick=\"window.open('/DetectionPolicy/rules/edit.cgi?"
           . "wizard=1&rule_uuid=$event->{rule_uuid}&rev_uuid=$event->{rule_rev}','newwindow',"
           . "'height=900, width=900, toolbar=no, menubar=no, scrollbars=yes, "
           . "resizeable=yes, location=no, directories=no, status=no')\">"
           . SF::i18n::get_pm_msg("Edit", "PacketView") . "</a><br>";
   }

   $strURL .= "<a class=data href=\"javascript:void(0)\" onClick=\"window.open('/events/context.cgi?"
       . "Type=Rules&behavior=documentation&uuid=$event->{rule_uuid}&rev_uuid=$event->{rule_rev}','newwindow',"
       . "'height=900, width=900, toolbar=no, menubar=no, scrollbars=yes, "
       . "resizeable=yes, location=no, directories=no, status=no')\">"
       . SF::i18n::get_pm_msg("View Documentation", "PacketView") . "</a>";

   if (SF::Permission::hasPermission("intrusion_policy.modify"))
   {
       $strURL .= "<br><a class=data href=\"javascript:void(0)\" onClick=\"window.open('/DetectionPolicy/rules/comment.cgi?"
           . "gen=$event->{gen}&sid=$event->{sid}&rev=$event->{rev}&from=packet&nomenu=1&notitle=1','newwindow',"
           . "'height=900, width=900, toolbar=no, menubar=no, scrollbars=yes, "
           . "resizeable=yes, location=no, directories=no, status=no')\">"
           . SF::i18n::get_pm_msg("Rule Comment", "PacketView") . "</a>";
   }

   return $strURL;
}


sub draw_packet
{
    my ( $packet, $var ) = @_;
    warn "draw_packet ** ". Dumper( $packet ) if ( $_debug && $_debug > 2 );

    my $frameData = draw_frame( $packet, $var );
    my $payloadData = draw_payload( $packet, {
            frame_id => $frameData->{frame_id},
            section_id => $frameData->{section_id},
            }, $var
        );

    my $frameID = $frameData->{frame_id};
    my $frameCount = $frameData->{frame_count};
    my $displayFrame = $frameData->{display_text};
    my $displayPayload = $payloadData->{display_text};
    my $ExpandAllStr = SF::i18n::get_pm_msg( "expand_all", "PacketView");
    my $expand = !$var || $var->{outputType} eq 'HTML' ? "<a class=iconlink id='$frameID' onclick='expandFrame( this, \"$frameID\" )'>$ExpandAllStr</a>" : "";
    my $html =<<EOF;

<div class=side-bar style='display:block; margin-bottom: 8px'>
  <h3 class=frame-header>FRAME $frameCount $expand</h3>
$displayFrame
$displayPayload
</div>
EOF
    if ( $expandState && $expandState == 2 )
    {
        $html .= "<script> expandSection('$frameID')</script>";
    }

    return $html;
}

sub draw_frame
{
    my ( $packet, $var ) = @_;
    warn " **draw_frame\n" if ( $_debug && $_debug > 2 );
    warn "Expand State $expandState\n" if ( $_debug );
    my @lines = @{$packet->{data}};
    my $retStruct;
    my $dc = $packet->{dc};
    my $frameCount = 1;
    $frameCount = $1 if ($lines[0] =~ /Frame (\d+)/);
    my $frameID = $packet->{eventCount} ."_" . $frameCount;
    my $section_id = 0;
    my $in_tbody;
    my $lineCount = @lines;
    my $html = "";
    for( my $pos = 0; $pos < $lineCount; $pos++ )
    {
        my $line = SF::UIUtils::encodeHTML($lines[$pos]);
        if ( $pos == 0)
        {
            $line =~ s/\(.*,\s*/\(/;
        }
        else
        {
            $line =~ s{ (\d+\.\d+\.\d+\.\d+)\s+\((\d+\.\d+\.\d+\.\d+)\) } { SF::Render::render("packetview.ip", $1, $dc) }gex;
            $line =~ s{ (\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\s+\((\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\) } { SF::Render::render("packetview.macaddr", $1) }gex;
        }

        if($line =~ /^(.*Time.*\s+)(\w+\s+\d+,\s+\d+\s+\d+:\d+:\d+)(\.\d+)?(.*)$/)
        {
            ## We add the TZ Offset because UnixDate assumes the time that is passed in is in the local TZ
            ## This will essentially get the correct UTC time. Local time will do the correct TZ conversions
            my $time = UnixDate($2, "%s");
            $line = $1 . strftime("%b %d, %Y %X", localtime($time) ) . $3;
        }

        if( $line =~ /^\s+/ )
        {
            my ($label, $value) = $line =~ /^\s*([\w\s]+)\s*:\s*(.*)$/;
            if( $pos+1 < $lineCount && $lines[$pos+1] =~ /^\s{5}/ )
            {
                # begins a sub section
                $html .= "<tr><th align=left>$label</th><td>";
                # walk subsections
                $html .= "<pre>";
                for( ; (($pos+1 < $lineCount) && ($lines[$pos+1] =~ /^\s{5}/)); $pos++ )
                {
                    my $newline = SF::UIUtils::encodeHTML($lines[$pos+1]);
                    $newline =~ s/^\s*//;
                    $newline =~ s{ (\d+\.\d+\.\d+\.\d+)\s+\((\d+\.\d+\.\d+\.\d+)\) } { SF::Render::render("packetview.ip", $1, $dc) }gex;
                    $newline =~ s{ (\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\s+\((\w\w:\w\w:\w\w:\w\w:\w\w:\w\w)\) } { SF::Render::render("packetview.macaddr", $1) }gex;
                    $html .= "$newline\n";
                }
                $html .= "</pre></td></tr>";
            }
            else
            {
                # normal section
                $html .= "<tr><th align=left>$label</th><td>$value</td></tr>";
            }
        }
        else
        {
            if ($line =~ /,/)
            {
                $line =~ s/,\s*/ <span class="light">(/;
                $line .= ')</span>';
            }
            else
            {
                $line =~ s/\((.*)\)\s*$/ <span class="light">($1)/;
            }
            $html .= "</table></div>" if $in_tbody;
            $section_id++;
            if ($var->{outputType} eq "HTML") {
                $html .= "<div class='collapsed section' id=section_${frameID}_${section_id}>";
                $html .= "<h3 onclick=toggleSection('section_${frameID}_${section_id}')>$line</h3>";
            }
            else {
                $html .= "<div class='section'>";
                $html .= "<h3>$line</h3>";
            }
            $html .= "<table class='simple-table content'>";
            $in_tbody = 1;
        }
    }
    $html .= "</table></div>" if $in_tbody;
    $section_id++;
    $retStruct->{frame_count} = ${frameCount};
    $retStruct->{frame_id} = ${frameID};
    $retStruct->{section_id} = ${section_id};
    $retStruct->{display_text} = $html;
    return $retStruct;
}

sub convertText
{
    my ( $hexText) = @_;
    #
    # Convert hex into unsigned char, and replace no-printable character with .
    #
    $hexText =~ s/\s+//g;
    $hexText =~ s/([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
    $hexText =~ s/[^\n\r[:print:]]/./g;
    return SF::UIUtils::encodeHTML( $hexText );
}

sub draw_payload
{
    my ( $packet, $options, $var ) = @_;
    warn " **draw_payload\n". Dumper( $options ) if ( $_debug && $_debug > 2 );

    my @lines = @{$packet->{payload}};
    my $html;
    my $frameID = $options->{frame_id};
    my $sectionID = $options->{section_id};

    my @toExpand;
    my @payloadStruct;
    my $asBold = 0;
    my $htmlPacket;
    my $htmlData;
    my $numSection;
    my $hexSection;
    my $ascSection;
    my $dataSection;
    my $completePacket;

    # No Section Titles
    if ( $lines[0] =~ /^\S{4,}\s+/ )
    {
        foreach my $line ( @lines )
        {
            my ( $num, $hex, $asc ) = split( /\s\s+/, $line, 3);
            $dataSection .= $hex;
            $numSection .= SF::UIUtils::encodeHTML($num);
            $hexSection .= SF::UIUtils::encodeHTML($hex);
            $ascSection .= SF::UIUtils::encodeHTML($asc);
            $completePacket .= '<tr><td><pre>'.SF::UIUtils::encodeHTML($num).'</pre></td>'.
                                   '<td style="color:gray;"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($hex).'</pre></td>'.
                                   '<td style="color:gray;"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($asc).'</pre></td>'.
                               '</tr>';
        }

        $dataSection = convertText( $dataSection );
        $htmlData = "<pre>$dataSection</pre>";
        $htmlPacket = "<table>\n\t$completePacket\n</table>";
    }

    # Section Titles
    else
    {
        my $sectionTitle;
        foreach my $line ( @lines )
        {
            next if ( !$line );
            # Packet Data
            if ( $line =~ /^\S{4,}\s+/ )
            {
                my ( $num, $hex, $asc ) = split( /\s\s+/, $line, 3);
                $dataSection .= $hex;
                $numSection .= SF::UIUtils::encodeHTML($num);
                $hexSection .= SF::UIUtils::encodeHTML($hex);
                $ascSection .= SF::UIUtils::encodeHTML($asc);
                $completePacket .= '<tr><td><pre>'.SF::UIUtils::encodeHTML($num).'</pre></td>'.
                                       '<td style="color:gray"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($hex).'</pre></td>'.
                                       '<td style="color:gray;"><pre>&nbsp;&nbsp;'.SF::UIUtils::encodeHTML($asc).'</pre></td>'.
                                   '</tr>';
            }
            # Section Title 
            else
            {
                if ( $sectionTitle )
                {
                    $dataSection = convertText( $dataSection );
                    $htmlData .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
                      <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
                      <div class=content><pre>$dataSection</pre></div>
                    </div>";

                    push @toExpand, $sectionID if ( $expandState && $expandState == 3 );
                    $sectionID++;
                    $htmlPacket .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
                      <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
                      <div class=content><table>$completePacket</table></div>
                    </div>";

                    push @toExpand, $sectionID if ( $expandState && $expandState == 1 );
                    $sectionID++;
                }
                $sectionTitle = $line;
                $numSection = "";
                $hexSection = "";
                $ascSection = "";
                $completePacket = "";
                $dataSection = "";
            }
        }

        if ( $sectionTitle )
        {
            $dataSection = convertText( $dataSection );
            $htmlData .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
              <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
              <div class=content><pre>$dataSection</pre></div>
            </div>";

            push @toExpand, $sectionID if ( $expandState && $expandState == 3 );
            $sectionID++;
            $htmlPacket .= "<div class='collapsed section' id=section_${frameID}_${sectionID}>
              <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$sectionTitle</h3>
              <div class=content><table>$completePacket</table></div>
            </div>";

            push @toExpand, $sectionID if ( $expandState && $expandState == 1 );
            $sectionID++;
        }
    }

    #Force English text
    my $packetByte = SF::i18n::get_pm_msg("Packet Bytes", "PacketView", "en_US");
    my $packetData = SF::i18n::get_pm_msg("Packet Text", "PacketView", "en_US");

    if ($var->{outputType} eq "HTML") {
        $html =<<EOF;
<div class='collapsed section' id=section_${frameID}_${sectionID}>
  <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$packetData</h3>
  <div class=content>$htmlData</div>
</div>
EOF
    }
    else {
        $html =<<EOF;
<div class='section'>
  <h3>$packetData</h3>
  <div class=content>$htmlData</div>
</div>
EOF
    }

    push @toExpand, $sectionID if ( $expandState && $expandState == 3 );
    $sectionID++;

    if ($var->{outputType} eq "HTML") {
        $html .=<<EOF;
<div class='collapsed section' id=section_${frameID}_${sectionID}>
  <h3 onclick=toggleSection('section_${frameID}_${sectionID}')>$packetByte</h3>
  <div class=content>$htmlPacket</div>
</div>
EOF
    }
    else {
        $html .=<<EOF;
<div class='section'>
  <h3>$packetByte</h3>
  <div class=content>$htmlPacket</div>
</div>
EOF
    }

    push @toExpand, $sectionID if ( $expandState && $expandState == 1 );
    $sectionID++;
    if ( $expandState && scalar(@toExpand) )
    {
        foreach my $idx ( @toExpand )
        {
            $html .= "<script> expandSection('img_${frameID}_${idx}')</script>";
        }
    }
    $options->{section_id} = $sectionID;

    my $retStruct;
    $retStruct->{display_text} = $html;
    return $retStruct;
}

sub preRenderPortScan
{
    my ( $frame, $addDSTInfo, $eventInfo ) = @_;
    $frame->{payload} =~ s/\s//g;
    my $len = $frame->{DATALen} * 2;
    my $dc = $eventInfo->{'sensor'};
    $len = 0 - $len;
    my $info = substr( $frame->{payload}, $len );
    $info =~ s/([a-fA-F0-9][a-fA-F0-9])/pack("C",hex($1))/eg;
    if (( $info =~ /Open Port:/ ) && $addDSTInfo )
    {
        my $dstAddr = $frame->{DSTAddress};
        $info =~ s/Open Port:\s*/Open Port: $dstAddr /g;
    }
    $info =~ s{ (\d+\.\d+\.\d+\.\d+) } { SF::Render::render("packetview.ip", $1, $dc) }gex;

    $info =~ s{ (.*):\s } { SF::i18n::get_pm_msg("$1", "PacketView").": " }gex;
    $frame->{display_text} = $info;
}

sub processPortscan
{
    my ( $var, $eventInfo ) = @_;
    my $inPayload = 0;
    my $frameCount = 0;
    my $frameData;
    foreach my $line ( @{$var->{port_scan_data}} )
    {
        if ( $inPayload == 1)
        {
            if( $line =~ /^$/ )
            {
                preRenderPortScan( $frameData, $var->{addDSTInfo}, $eventInfo);
                $inPayload = 0;
            }
            else
            {
                $frameData->{payload} .= substr($line, 6, 48);
            }
        }
        elsif ( $line =~ /^$/ )
        {
            $inPayload = 1;
            $frameData->{payload} = "";
        }
        elsif ( $line =~ /^Frame/ )
        {
            $frameData = {};
            $var->{data}[$frameCount++] = $frameData;
            $inPayload = 0;
        }
        elsif ( $line =~ /^Internet Protocol/ )
        {

            $line =~ /\((.+)\).*\((.+)\)/;
            $frameData->{SRCAddress} = $1;
            $frameData->{DSTAddress} = $2;
        }
        elsif ( $line =~ /^Data/ )
        {
            $line =~ /Data .(\d+)/;
            $frameData->{DATALen} = $1;
        }
    }
    preRenderPortScan( $frameData, $var->{addDSTInfo}, $eventInfo ) if ( !$frameData->{display_text} );
    warn Dumper( $eventInfo ) if ( $_debug && $_debug > 2 );
    warn Dumper( $var->{data} ) if ( $_debug && $_debug > 2 );
    push @{$var->{display_list}}, "<tr><th class=title>"
        . SF::i18n::get_pm_msg("Portscan Information", "PacketView") . "</th></tr>";

    my  $deName = SF::i18n::get_pm_msg("Detection Engine", "PacketView") . ":"
        . SF::Render::render( 'detection_engine_id', $eventInfo->{'sensor'} );
    $deName .= " (Software Agent) " if ( SF::Sensor::isAgentDE( split /:/, $eventInfo->{'sensor'} ));

    my $displayText;
    push @{$displayText}, $deName;
    push @{$displayText}, SF::i18n::get_pm_msg("Time", "PacketView") . ":"
        . SF::Render::render( 'time', $eventInfo->{tvsec} );
    push @{$displayText}, SF::i18n::get_pm_msg("Message", "PacketView") . ":"
        . $eventInfo->{display_text};

    my $idx;
    for ($idx = 0; $idx < scalar( @{$var->{data}} ); $idx++ )
    {
        my $frameData = $var->{data}[$idx];
        if ( $idx == 0 )
        {
            push @{$displayText}, SF::i18n::get_pm_msg("Source IP", "PacketView"). ":"
                . SF::Render::render("packetview.ip", $frameData->{SRCAddress}, $eventInfo->{'sensor'});
            push @{$displayText}, SF::i18n::get_pm_msg("Destination IP", "PacketView"). ":"
                . SF::Render::render("packetview.ip", $frameData->{DSTAddress}, $eventInfo->{'sensor'});
        }
        push @{$displayText}, split( "\n", $frameData->{display_text} );
    }
    warn Dumper( $displayText ) if ( $_debug && $_debug > 2 );

    for ( $idx = 0; $idx < scalar( @{$displayText} ); $idx++ )
    {
        my ($head, $text) = split( /\s*:\s*/, $displayText->[$idx], 2 );
        push @{$var->{display_list}}, "<tr><td> <b>$head :</b> $text</td></tr>";
    }
    return 1;
}

1;
