#!/bin/bash 

# Quick installer for the FPC patch on a Sourcefire DC 
# Adopted from initial work by leon.ward@sourcefire.com 2010,2011,2012,2013
# updated to install Endace Files

PATCH_NAME="Endace Patch"
PATCH_ID="Endace_Integration_Patch"
PATCH_VER=2.0
DC_VER=$(cat /etc/sf/ims.conf|grep SWVERSION | awk -F \= '{print $2}')
OS_VER=$(cat /etc/sf/ims.conf|grep OSVERSION | awk -F \= '{print $2}')
PERL_VER=$(cat /etc/sf/ims.conf |grep PERLVERSION | awk -F \= '{print $2}')
FILES_TO_PATCH="/usr/local/sf/lib/perl/$PERL_VER/SF/PacketView.pm" 	# Only one here for now, but I may add a context menu in future
CPFILES="/Volume/$OS_VER/sf/htdocs/img/icons/medium/endace_logo.png /Volume/$OS_VER/sf/htdocs/img/icons/medium/endace_vision_logo.png /etc/sf/endace.conf /etc/sf/endace_viz_id.conf"
CONFIG="/etc/sf/endace.conf"
IAM=$(whoami)
NOW=$(date +%s)
FORCE=$2



function die() {
    echo $1
    exit 1
}

function chk_compatible(){
	compatible=1

	[ "$FORCE" == "force" ] && return 0

	for i in "$FILES_TO_PATCH"
	do
 		PATCH=$(basename $i).patch   # PacketView.pm.patch
                patch -s -f --dry-run $i $PATCH > /dev/null || compatible=0
		echo -e "  Patch $PATCH will clean apply to $i"
	done
	
	if [ $compatible == 1 ] 
	echo $compatible
	then
		echo "* Looks like patch is good for $DC_VER"
	else 
		echo "* FAIL - Patch doesn't apply clean, won't install"
		exit 1
	fi
}

function chk_root() {
    if [ "$IAM" != "root" ]
    then
        die "! You must be root to install this patch"
    fi  
}

function do_config(){
	#echo -e "* Configuration"
        #echo -e "  To configure this patch, please edit the file $CONFIG\n"

	echo -e "* Configuration"
	
	valid=0
	while [ $valid -eq 0 ]; do
		read -p "Enter the Sourcefire DE ID: " DE_MAP_ID
		if [ "$DE_MAP_ID" -eq "$DE_MAP_ID" 2> /dev/null ]; then
			valid=1
		else
			echo "The input must be a positive integer"
			valid=0
		fi
	done
	
	
	read -p "Enter the IP address for the Endace probe: " Endace_Probe_IP
	
	read -p "Enter the User Login for the Endace probe: " EP_User
	
	read -p "Enter the User Passwd for the Endace probe: " EP_Pass
	
	valid=0
	while [ $valid -eq 0 ]; do
		read -p "Enter the number of seconds before the incident's time: " before_time
		if [ "$before_time" -eq "$before_time" 2> /dev/null ]; then
			valid=1
		else
			echo "The input must be a positive integer"
			valid=0
		fi
	done

	valid=0
	while [ $valid -eq 0 ]; do
		read -p "Enter the number of seconds after the incident's time: " after_time
		if [ "$after_time" -eq "$after_time" 2> /dev/null ]; then
			valid=1
		else
			echo "The input must be a positive integer"
			valid=0
		fi
	done
	
	valid=0
	while [ $valid -eq 0 ]; do
		read -p "Enter the number of seconds after which the old datamines will be purged from Endace probe: " Datamine_created_before
		if [ "$Datamine_created_before" -eq "$Datamine_created_before" 2> /dev/null ]; then
			valid=1
		else
			echo "The input must be a positive integer"
			valid=0
		fi
	done

	#echo -e "#DE_MAP_ID,Endace_Probe_IP,EP_User,EP_Pass,Seconds_Before,Seconds_After,Datamine_created_before" > $CONFIG
	echo -e "$DE_MAP_ID,$Endace_Probe_IP,$EP_User,$EP_Pass,$before_time,$after_time,$Datamine_created_before " >> $CONFIG
}
	


function do_status() {
	echo -e "* Status"
	for i in $FILES_TO_PATCH
	do
		if grep "$PATCH_ID" $i > /dev/null
		then
			echo -e "  Yes $i is patched"
		else
			echo -e "  NO  $i is NOT patched"
		fi
	done
        
        for i in $CPFILES
        do
            if [ -f $i ]
            then
                echo -e "- INSTALLED $i"
            else
                echo -e "- NOT INSTALLED $i"
            fi
        done

	echo "* Checking compatability with current release..."
	chk_compatible
}

function do_install(){
	echo -e "* Patching with $PATCH_NAME...."


	for i in $FILES_TO_PATCH
	do
		if grep "$PATCH_ID" $i > /dev/null
		then
			echo "X Not applying patch against $i - Already patched"
		else
			chk_compatible
			# Take a backup of the non-patched file
			BACKUPFILE=$(basename $i).backup.VER-$DC_VER.$NOW
			echo - Taking backup of $i $BACKUPFILE
			cp $i $BACKUPFILE
			PATCH=$(basename $i).patch   # PacketView.pm.patch
			patch -s -f $i $PATCH
		fi
	done	

	echo -e "* Copy files"

	for i in $CPFILES
	do
		FILE=$(basename $i)
		DIR=$(dirname $i)
		echo -e "- Installing $i"
		if [ -f $FILE ] ; then
			cp $FILE $i
			if [ $FILE == "endace_viz_id.conf" ] ; then
				chmod 666 $i
			fi

		else 
			echo -e "X Error - Cant find FILE"
		fi
	done

}

function do_remove(){
	echo -e "* Removing $PATCH_NAME...."

        for i in $FILES_TO_PATCH
        do
                if grep "$PATCH_ID" $i > /dev/null
                then
                        PATCH=$(basename $i).patch
                        echo "  Removing - $PATCH from $i"
                        patch -sR $i $PATCH
		else
			echo "X Unpatched - $i . Nothing to do "
                fi
        done

	echo -e "* Remove files...."

	for i in $CPFILES
	do
		echo -e "  Removing $i"
		if [ -f $i ] ; then
			rm $i
		else 
			echo -e "X NOT INSTALLED - Cant find $i"
		fi
	done

}



################################################################################

echo -e "\n ***************************************************"
echo -e " * $PATCH_NAME v$PATCH_VER for Sourcefire Defense Center *"
echo -e " ***************************************************"

chk_root

case $1 in 
	install)
		do_install
	;;
	remove)
		do_remove
	;;
	status)
		do_status
	;;
	reapply)
		do_remove
		do_install
	;;
	config)
		do_config
	;;
	*)
		echo -e "\n * Usage"
		echo -e "  $0 <install | remove | status | reapply | config> | <force>\n"
		echo -e "  install -> Install the patch"
		echo -e "  remove  -> Uninstall the patch"
		echo -e "  status  -> Check status of patched files"
		echo -e "  reapply -> Re-apply patch"
		echo -e "  config  -> Configure $CONFIG"
		echo -e "  force   -> Ignore version checks and install/remove anyway"
		echo

esac

